@@ -25,6 +25,7 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include <glib/gstdio.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-bindings.h>
 #include <dbus/dbus-glib-lowlevel.h>
@@ -54,23 +55,29 @@ enum
   PROP_STYLESHEET,
   PROP_BUTTON_STYLE,
   PROP_XID,
-
-  PROP_TOOLBAR_SERVICE
+  PROP_TOOLBAR_SERVICE,
+  PROP_DELAYED_READY
 };
 
 enum
 {
+  UNLOAD,
   SET_SIZE,
+  SET_POSITION,
+  SHOW,
   SHOW_BEGIN,
   SHOW_END,
+  HIDE,
   HIDE_BEGIN,
   HIDE_END,
 
-  REQUEST_SHOW,
-  REQUEST_HIDE,
   REQUEST_FOCUS,
   REQUEST_BUTTON_STYLE,
   REQUEST_TOOLTIP,
+  REQUEST_BUTTON_STATE,
+  REQUEST_MODALITY,
+
+  READY,
 
   LAST_SIGNAL
 };
@@ -89,13 +96,18 @@ struct _MplPanelClientPrivate
   gchar           *button_style;
   guint            xid;
 
+  gint             x;
+  gint             y;
   guint            width;
   guint            max_height;
   guint            requested_height;
   guint            real_height;
 
-  gboolean         constructed     : 1; /*poor man's constructor return value*/
-  gboolean         toolbar_service : 1;
+  gboolean         constructed       : 1; /*poor man's constr return value*/
+  gboolean         toolbar_service   : 1;
+  gboolean         ready_emitted     : 1;
+  gboolean         delayed_ready     : 1;
+  gboolean         main_loop_running : 1;
 };
 
 static void
@@ -126,6 +138,9 @@ mpl_panel_client_get_property (GObject    *object,
     case PROP_TOOLBAR_SERVICE:
       g_value_set_boolean (value, priv->toolbar_service);
       break;
+    case PROP_DELAYED_READY:
+      g_value_set_boolean (value, priv->delayed_ready);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -137,7 +152,8 @@ mpl_panel_client_set_property (GObject      *object,
                                const GValue *value,
                                GParamSpec   *pspec)
 {
-  MplPanelClientPrivate *priv = MPL_PANEL_CLIENT (object)->priv;
+  MplPanelClient        *panel = MPL_PANEL_CLIENT (object);
+  MplPanelClientPrivate *priv  = panel->priv;
 
   switch (property_id)
     {
@@ -163,6 +179,9 @@ mpl_panel_client_set_property (GObject      *object,
     case PROP_TOOLBAR_SERVICE:
       priv->toolbar_service = g_value_get_boolean (value);
       break;
+    case PROP_DELAYED_READY:
+      mpl_panel_client_set_delayed_ready (panel, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -212,6 +231,8 @@ mpl_panel_client_finalize (GObject *object)
  */
 static gboolean
 mnb_panel_dbus_init_panel (MplPanelClient  *self,
+                           gint             x,
+                           gint             y,
                            guint            width,
                            guint            height,
                            gchar          **name,
@@ -226,7 +247,7 @@ mnb_panel_dbus_init_panel (MplPanelClient  *self,
   MplPanelClientPrivate *priv = self->priv;
   guint real_height = height;
 
-  g_debug ("%s called", __FUNCTION__);
+  g_debug ("dbus init: %d,%d;%dx%d", x, y, width, height);
 
   if (!priv->xid)
     return FALSE;
@@ -237,6 +258,8 @@ mnb_panel_dbus_init_panel (MplPanelClient  *self,
   *stylesheet   = g_strdup (priv->stylesheet);
   *button_style = g_strdup (priv->button_style);
 
+  priv->x          = x;
+  priv->y          = y;
   priv->max_height = height;
   priv->width      = width;
 
@@ -254,8 +277,25 @@ mnb_panel_dbus_init_panel (MplPanelClient  *self,
   *alloc_width  = width;
   *alloc_height = real_height;
 
+  /*
+   * Make sure that the window is hidden (the window can be left mapped, if the
+   * Toolbar died on us, and then bad things happen).
+   */
+  mpl_panel_client_hide (self);
+
+  g_signal_emit (self, signals[SET_POSITION], 0, x, y);
   g_signal_emit (self, signals[SET_SIZE], 0, width, real_height);
 
+  if (priv->ready_emitted)
+    {
+      /*
+       * Are getting embedded for a second, or n-th time; emit the ready
+       * signal.
+       */
+
+      g_signal_emit (self, signals[READY], 0);
+    }
+
   return TRUE;
 }
 
@@ -300,6 +340,37 @@ mnb_panel_dbus_set_size (MplPanelClient  *self,
   return TRUE;
 }
 
+/*
+ * The functions required by the interface.
+ */
+static gboolean
+mnb_panel_dbus_set_position (MplPanelClient  *self,
+                             gint             x,
+                             gint             y,
+                             GError         **error)
+{
+  MplPanelClientPrivate *priv = self->priv;
+  guint old_x = priv->x;
+  guint old_y = priv->y;
+
+  g_debug ("%s called: x %d (%d), y %d (%d)",
+           __FUNCTION__, x, priv->x, y, priv->y);
+
+
+  if (old_x != priv->x || old_y != priv->y)
+    g_signal_emit (self, signals[SET_POSITION], 0, x, y);
+
+  return TRUE;
+}
+
+static gboolean
+mnb_panel_dbus_show (MplPanelClient *self, GError **error)
+{
+  g_debug ("%s called", __FUNCTION__);
+  g_signal_emit (self, signals[SHOW], 0);
+  return TRUE;
+}
+
 static gboolean
 mnb_panel_dbus_show_begin (MplPanelClient *self, GError **error)
 {
@@ -316,6 +387,14 @@ mnb_panel_dbus_show_end (MplPanelClient *self, GError **error)
   return TRUE;
 }
 
+static gboolean
+mnb_panel_dbus_hide (MplPanelClient *self, GError **error)
+{
+  g_debug ("%s called", __FUNCTION__);
+  g_signal_emit (self, signals[HIDE], 0);
+  return TRUE;
+}
+
 static gboolean
 mnb_panel_dbus_hide_begin (MplPanelClient *self, GError **error)
 {
@@ -339,12 +418,28 @@ mnb_panel_dbus_ping (MplPanelClient *self, GError **error)
   return TRUE;
 }
 
+static gboolean
+mnb_panel_dbus_unload (MplPanelClient *self, GError **error)
+{
+  g_debug ("%s called", __FUNCTION__);
+  g_signal_emit (self, signals[UNLOAD], 0);
+  return TRUE;
+}
+
 #include "mnb-panel-dbus-glue.h"
 
+static gboolean
+mpl_panel_client_real_unload (MplPanelClient *panel)
+{
+  g_main_loop_quit (NULL);
+
+  return TRUE;
+}
+
 static void
 mpl_panel_client_class_init (MplPanelClientClass *klass)
 {
-  GObjectClass     *object_class   = G_OBJECT_CLASS (klass);
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (MplPanelClientPrivate));
 
@@ -354,6 +449,8 @@ mpl_panel_client_class_init (MplPanelClientClass *klass)
   object_class->finalize         = mpl_panel_client_finalize;
   object_class->constructed      = mpl_panel_client_constructed;
 
+  klass->unload                  = mpl_panel_client_real_unload;
+
   dbus_g_object_type_install_info (G_TYPE_FROM_CLASS (klass),
                                    &dbus_glib_mnb_panel_dbus_object_info);
 
@@ -399,7 +496,7 @@ mpl_panel_client_class_init (MplPanelClientClass *klass)
                                                       "XID",
                                                       "XID",
                                                       0, G_MAXUINT,
-                                                      1024,
+                                                      0,
                                                       G_PARAM_READWRITE |
                                                       G_PARAM_CONSTRUCT));
 
@@ -413,6 +510,25 @@ mpl_panel_client_class_init (MplPanelClientClass *klass)
                                      G_PARAM_READWRITE |
                                      G_PARAM_CONSTRUCT_ONLY));
 
+  g_object_class_install_property (object_class,
+                                   PROP_DELAYED_READY,
+                                   g_param_spec_boolean ("delayed-ready",
+                                     "Delayed emission of 'ready' signal",
+                                     "Whether emission of 'ready' signal should"
+                                     " be delayed until later.",
+                                     FALSE,
+                                     G_PARAM_READWRITE |
+                                     G_PARAM_CONSTRUCT));
+
+  signals[UNLOAD] =
+    g_signal_new ("unload",
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (MplPanelClientClass, unload),
+                  NULL, NULL,
+                  moblin_netbook_marshal_BOOLEAN__VOID,
+                  G_TYPE_BOOLEAN, 0);
+
   signals[SET_SIZE] =
     g_signal_new ("set-size",
                   G_TYPE_FROM_CLASS (object_class),
@@ -424,6 +540,17 @@ mpl_panel_client_class_init (MplPanelClientClass *klass)
                   G_TYPE_UINT,
                   G_TYPE_UINT);
 
+  signals[SET_POSITION] =
+    g_signal_new ("set-position",
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (MplPanelClientClass, set_position),
+                  NULL, NULL,
+                  moblin_netbook_marshal_VOID__INT_INT,
+                  G_TYPE_NONE, 2,
+                  G_TYPE_INT,
+                  G_TYPE_INT);
+
   signals[SHOW_BEGIN] =
     g_signal_new ("show-begin",
                   G_TYPE_FROM_CLASS (object_class),
@@ -460,20 +587,20 @@ mpl_panel_client_class_init (MplPanelClientClass *klass)
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
-  signals[REQUEST_SHOW] =
-    g_signal_new ("request-show",
+  signals[SHOW] =
+    g_signal_new ("show",
                   G_TYPE_FROM_CLASS (object_class),
                   G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (MplPanelClientClass, request_show),
+                  G_STRUCT_OFFSET (MplPanelClientClass, show),
                   NULL, NULL,
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
-  signals[REQUEST_HIDE] =
-    g_signal_new ("request-hide",
+  signals[HIDE] =
+    g_signal_new ("hide",
                   G_TYPE_FROM_CLASS (object_class),
                   G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (MplPanelClientClass, request_hide),
+                  G_STRUCT_OFFSET (MplPanelClientClass, hide),
                   NULL, NULL,
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
@@ -506,14 +633,49 @@ mpl_panel_client_class_init (MplPanelClientClass *klass)
                   g_cclosure_marshal_VOID__STRING,
                   G_TYPE_NONE, 1,
                   G_TYPE_STRING);
+
+  signals[REQUEST_BUTTON_STATE] =
+    g_signal_new ("request-button-state",
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (MplPanelClientClass, request_button_state),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__ENUM,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_ENUM);
+
+  signals[REQUEST_MODALITY] =
+    g_signal_new ("request-modality",
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (MplPanelClientClass, request_modality),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__BOOLEAN,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_BOOLEAN);
+
+  signals[READY] =
+    g_signal_new ("ready",
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 }
 
 static void
 mpl_panel_client_init (MplPanelClient *self)
 {
   MplPanelClientPrivate *priv;
+  const gchar           *home;
 
   priv = self->priv = MPL_PANEL_CLIENT_GET_PRIVATE (self);
+
+  home = g_get_home_dir ();
+
+  if (g_chdir (home))
+    g_warning ("Unable to change working directory to '%s'", home);
 }
 
 static DBusGConnection *
@@ -605,6 +767,8 @@ mpl_panel_client_noc_cb (DBusGProxy     *proxy,
        */
       g_debug ("Toolbar gone away, cleaning up");
       priv->toolbar_proxy = NULL;
+
+      mpl_panel_client_hide (panel);
       return;
     }
 
@@ -702,6 +866,46 @@ mpl_panel_client_setup_toolbar_proxy (MplPanelClient *panel)
   return TRUE;
 }
 
+/*
+ * We install a one-off idle to emit the READY signal
+ */
+static gboolean
+mpl_panel_client_ready_idle_cb (gpointer data)
+{
+  MplPanelClient *panel = data;
+
+  static guint count = 0;
+
+  /*
+   * Allow the main loop to spin few times
+   */
+  if (count++ < 5)
+    return TRUE;
+
+  g_signal_emit (panel, signals[READY], 0);
+
+  panel->priv->ready_emitted = TRUE;
+
+  return FALSE;
+}
+
+/*
+ * We install a one-off idle to install the ready idle, if needed
+ */
+static gboolean
+mpl_panel_client_one_off_idle_cb (gpointer data)
+{
+  MplPanelClient *panel = data;
+
+  if (!panel->priv->delayed_ready)
+    g_idle_add_full (G_PRIORITY_LOW,
+                     mpl_panel_client_ready_idle_cb, panel, NULL);
+
+  panel->priv->main_loop_running = TRUE;
+
+  return FALSE;
+}
+
 static void
 mpl_panel_client_constructed (GObject *self)
 {
@@ -770,6 +974,8 @@ mpl_panel_client_constructed (GObject *self)
    * plan.
    */
   priv->constructed = TRUE;
+
+  g_idle_add (mpl_panel_client_one_off_idle_cb, self);
 }
 
 MplPanelClient *
@@ -798,15 +1004,21 @@ mpl_panel_client_new (guint        xid,
 }
 
 void
-mpl_panel_client_request_show (MplPanelClient *panel)
+mpl_panel_client_unload (MplPanelClient *panel)
 {
-  g_signal_emit (panel, signals[REQUEST_SHOW], 0);
+  g_signal_emit (panel, signals[UNLOAD], 0);
 }
 
 void
-mpl_panel_client_request_hide (MplPanelClient *panel)
+mpl_panel_client_show (MplPanelClient *panel)
 {
-  g_signal_emit (panel, signals[REQUEST_HIDE], 0);
+  g_signal_emit (panel, signals[SHOW], 0);
+}
+
+void
+mpl_panel_client_hide (MplPanelClient *panel)
+{
+  g_signal_emit (panel, signals[HIDE], 0);
 }
 
 void
@@ -842,6 +1054,19 @@ mpl_panel_client_request_tooltip (MplPanelClient *panel,
   g_signal_emit (panel, signals[REQUEST_TOOLTIP], 0, tooltip);
 }
 
+void
+mpl_panel_client_request_button_state (MplPanelClient *panel,
+                                       MnbButtonState  state)
+{
+  g_signal_emit (panel, signals[REQUEST_BUTTON_STATE], 0, state);
+}
+
+void
+mpl_panel_client_request_modality (MplPanelClient *panel, gboolean modal)
+{
+  g_signal_emit (panel, signals[REQUEST_MODALITY], 0, modal);
+}
+
 #include "mnb-toolbar-dbus-bindings.h"
 
 /*
@@ -1085,3 +1310,84 @@ mpl_panel_client_get_xid (MplPanelClient *panel)
   return panel->priv->xid;
 }
 
+void
+mpl_panel_client_request_show (MplPanelClient *panel)
+{
+  g_warning ("%s is deprecated use mpl_panel_client_show() instead.",
+             __FUNCTION__);
+
+  mpl_panel_client_show (panel);
+}
+
+void
+mpl_panel_client_request_hide (MplPanelClient *panel)
+{
+  g_warning ("%s is deprecated use mpl_panel_client_hide() instead.",
+             __FUNCTION__);
+
+  mpl_panel_client_hide (panel);
+}
+
+void
+mpl_panel_client_ready (MplPanelClient *panel)
+{
+  MplPanelClientPrivate *priv;
+
+  g_return_if_fail (MPL_IS_PANEL_CLIENT (panel));
+
+  priv = panel->priv;
+
+  if (!priv->delayed_ready)
+    {
+      g_warning ("%s can only be called if the client object was constructed "
+                 "with 'delayed-ready' property set to TRUE.!",
+                 __FUNCTION__);
+      return;
+    }
+
+  priv->ready_emitted = TRUE;
+
+  g_signal_emit (panel, signals[READY], 0);
+}
+
+/**
+ * mpl_panel_client_set_delayed_ready
+ * @panel: a #MplPanelClient
+ * @delayed: boolean value for the delayed-ready property.
+ *
+ * Sets the value of the delayed-ready property. When this property is set to
+ * %TRUE the #MplPanelClient::ready signal will not be emitted until the panel
+ * application calls mpl_panel_client_ready().
+ *
+ * Note: The delayed-ready property can only be set before the main loop is
+ * started; calling this function once the main loop has been started will
+ * result in a critical warning.
+ *
+ * Return value: %TRUE if the call was successful.
+ */
+gboolean
+mpl_panel_client_set_delayed_ready (MplPanelClient *panel, gboolean delayed)
+{
+  MplPanelClientPrivate *priv;
+  gboolean               not_delayed;
+
+  g_return_val_if_fail (MPL_IS_PANEL_CLIENT (panel), FALSE);
+
+  priv = panel->priv;
+
+  not_delayed = !priv->delayed_ready;
+
+  if (not_delayed != delayed)
+    return TRUE;
+
+  if (priv->main_loop_running)
+    {
+      g_critical ("The delayed-ready property has to be set before starting "
+                  "the main loop, but main loop is already running.");
+      return FALSE;
+    }
+
+  priv->delayed_ready = delayed;
+
+  return TRUE;
+}
