@@ -28,6 +28,8 @@
 #include <glib-object.h>
 #include <X11/X.h>
 
+#include "mpl-panel-common.h"
+
 G_BEGIN_DECLS
 
 #define MPL_TYPE_PANEL_CLIENT mpl_panel_client_get_type()
@@ -63,22 +65,25 @@ typedef struct
   /*
    * Public signals -- connect to these from your panel.
    */
-  void (*set_size)           (MplPanelClient *panel, guint width, guint height);
-  void (*show_begin)         (MplPanelClient *panel);
-  void (*show_end)           (MplPanelClient *panel);
-  void (*hide_begin)         (MplPanelClient *panel);
-  void (*hide_end)           (MplPanelClient *panel);
+  gboolean (*unload)         (MplPanelClient *panel);
+  void     (*set_size)       (MplPanelClient *panel, guint width, guint height);
+  void     (*set_position)   (MplPanelClient *panel, gint x, gint y);
+  void     (*show)           (MplPanelClient *panel);
+  void     (*show_begin)     (MplPanelClient *panel);
+  void     (*show_end)       (MplPanelClient *panel);
+  void     (*hide)           (MplPanelClient *panel);
+  void     (*hide_begin)     (MplPanelClient *panel);
+  void     (*hide_end)       (MplPanelClient *panel);
 
   /*
    * Private signals
    * Signals for DBus -- these are the interface signals.
    */
-  void (*request_show)         (MplPanelClient *panel);
-  void (*request_hide)         (MplPanelClient *panel);
   void (*request_focus)        (MplPanelClient *panel);
   void (*request_button_style) (MplPanelClient *panel, const gchar *style);
   void (*request_tooltip)      (MplPanelClient *panel, const gchar *tooltip);
-
+  void (*request_button_state) (MplPanelClient *panel, MnbButtonState state);
+  void (*request_modality)     (MplPanelClient *panel, gboolean modal);
 } MplPanelClientClass;
 
 GType mpl_panel_client_get_type (void);
@@ -89,17 +94,23 @@ MplPanelClient *mpl_panel_client_new       (guint        xid,
                                             const gchar *stylesheet,
                                             const gchar *button_style);
 
+void  mpl_panel_client_unload               (MplPanelClient *panel);
+
 void  mpl_panel_client_set_height_request   (MplPanelClient *panel,
                                              guint           height);
 guint mpl_panel_client_get_height_request   (MplPanelClient *panel);
 
-void  mpl_panel_client_request_show         (MplPanelClient *panel);
-void  mpl_panel_client_request_hide         (MplPanelClient *panel);
+void  mpl_panel_client_show                 (MplPanelClient *panel);
+void  mpl_panel_client_hide                 (MplPanelClient *panel);
 void  mpl_panel_client_request_focus        (MplPanelClient *panel);
 void  mpl_panel_client_request_button_style (MplPanelClient *panel,
                                              const gchar    *style);
 void  mpl_panel_client_request_tooltip      (MplPanelClient *panel,
                                              const gchar    *tooltip);
+void  mpl_panel_client_request_button_state (MplPanelClient *panel,
+                                             MnbButtonState  state);
+void  mpl_panel_client_request_modality     (MplPanelClient *panel,
+                                             gboolean        modal);
 
 gboolean mpl_panel_client_launch_application   (MplPanelClient *panel,
                                                 const gchar    *app);
@@ -113,8 +124,12 @@ gboolean
 mpl_panel_client_launch_default_application_for_uri (MplPanelClient *panel,
                                                      const gchar    *uri);
 
-Window
-mpl_panel_client_get_xid (MplPanelClient *panel);
+Window mpl_panel_client_get_xid (MplPanelClient *panel);
+
+void mpl_panel_client_ready (MplPanelClient *panel);
+
+gboolean mpl_panel_client_set_delayed_ready (MplPanelClient *panel,
+                                             gboolean        delayed);
 
 G_END_DECLS
 
