@@ -23,6 +23,7 @@
  */
 
 #include <gdk/gdkx.h>
+#include <math.h>
 
 #include "mpl-panel-gtk.h"
 
@@ -41,13 +42,14 @@ enum
 struct _MplPanelGtkPrivate
 {
   GtkWidget *window;
+  GtkWidget *child;
 };
 
 static void
 mpl_panel_gtk_get_property (GObject    *object,
-                               guint       property_id,
-                               GValue     *value,
-                               GParamSpec *pspec)
+                            guint       property_id,
+                            GValue     *value,
+                            GParamSpec *pspec)
 {
   switch (property_id)
     {
@@ -58,9 +60,9 @@ mpl_panel_gtk_get_property (GObject    *object,
 
 static void
 mpl_panel_gtk_set_property (GObject      *object,
-                               guint         property_id,
-                               const GValue *value,
-                               GParamSpec   *pspec)
+                            guint         property_id,
+                            const GValue *value,
+                            GParamSpec   *pspec)
 {
   switch (property_id)
     {
@@ -81,10 +83,55 @@ mpl_panel_gtk_finalize (GObject *object)
   G_OBJECT_CLASS (mpl_panel_gtk_parent_class)->finalize (object);
 }
 
+static void
+mpl_panel_gtk_set_size (MplPanelClient *self, guint width, guint height)
+{
+  MplPanelGtkPrivate  *priv = MPL_PANEL_GTK (self)->priv;
+  MplPanelClientClass *p_class;
+
+  p_class = MPL_PANEL_CLIENT_CLASS (mpl_panel_gtk_parent_class);
+
+  gtk_window_resize (GTK_WINDOW (priv->window), width, height);
+
+  if (p_class->set_size)
+    p_class->set_size (self, width, height);
+}
+
+static void
+mpl_panel_gtk_set_position (MplPanelClient *self, gint x, gint y)
+{
+  MplPanelGtkPrivate  *priv = MPL_PANEL_GTK (self)->priv;
+  MplPanelClientClass *p_class;
+
+  p_class = MPL_PANEL_CLIENT_CLASS (mpl_panel_gtk_parent_class);
+
+  gtk_window_move (GTK_WINDOW (priv->window), x, y);
+
+  if (p_class->set_position)
+    p_class->set_position (self, x, y);
+}
+
+static void
+mpl_panel_gtk_show (MplPanelClient *self)
+{
+  MplPanelGtkPrivate *priv = MPL_PANEL_GTK (self)->priv;
+
+  gtk_widget_show (priv->window);
+}
+
+static void
+mpl_panel_gtk_hide (MplPanelClient *self)
+{
+  MplPanelGtkPrivate *priv = MPL_PANEL_GTK (self)->priv;
+
+  gtk_widget_hide (priv->window);
+}
+
 static void
 mpl_panel_gtk_class_init (MplPanelGtkClass *klass)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GObjectClass        *object_class = G_OBJECT_CLASS (klass);
+  MplPanelClientClass *client_class = MPL_PANEL_CLIENT_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (MplPanelGtkPrivate));
 
@@ -93,6 +140,11 @@ mpl_panel_gtk_class_init (MplPanelGtkClass *klass)
   object_class->dispose          = mpl_panel_gtk_dispose;
   object_class->finalize         = mpl_panel_gtk_finalize;
   object_class->constructed      = mpl_panel_gtk_constructed;
+
+  client_class->set_position     = mpl_panel_gtk_set_position;
+  client_class->set_size         = mpl_panel_gtk_set_size;
+  client_class->show             = mpl_panel_gtk_show;
+  client_class->hide             = mpl_panel_gtk_hide;
 }
 
 static void
@@ -103,56 +155,6 @@ mpl_panel_gtk_init (MplPanelGtk *self)
   priv = self->priv = MPL_PANEL_GTK_GET_PRIVATE (self);
 }
 
-static gboolean
-mpl_panel_gtk_window_delete_event_cb (GtkWidget *window,
-                                      GdkEvent  *event,
-                                      gpointer   data)
-{
-  /*
-   * Ensure the config window will be available.
-   */
-  g_debug ("delete-event: interupting");
-  gtk_widget_hide (window);
-
-  /*
-   * Returning TRUE here stops the widget from getting destroyed.
-   */
-  return TRUE;
-}
-
-static void
-mpl_panel_gtk_embedded_cb (GtkWidget *widget, GParamSpec *pspec, gpointer data)
-{
-  gboolean embedded;
-
-  g_object_get (widget, "embedded", &embedded, NULL);
-
-  if (embedded)
-    gtk_widget_show (widget);
-  else
-    gtk_widget_hide (widget);
-}
-
-/*
- * The size of the plug and the size of the backing X window tends to get
- * out of sync (MB#6246), so when the socket is resized, force the size on the
- * backing window.
- */
-static void
-mpl_panel_gtk_size_allocate_cb (GtkWidget     *widget,
-                                GtkAllocation *allocation,
-                                gpointer       data)
-{
-  g_debug ("Plug allocated %d,%d;%dx%d.",
-           allocation->x,
-           allocation->y,
-           allocation->width,
-           allocation->height);
-
-  if (widget->window)
-    gdk_window_resize (widget->window, allocation->width, allocation->height);
-}
-
 static void
 mpl_panel_gtk_constructed (GObject *self)
 {
@@ -162,19 +164,15 @@ mpl_panel_gtk_constructed (GObject *self)
   if (G_OBJECT_CLASS (mpl_panel_gtk_parent_class)->constructed)
     G_OBJECT_CLASS (mpl_panel_gtk_parent_class)->constructed (self);
 
-  priv->window = window = gtk_plug_new (0);
+  priv->window = window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_type_hint (GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_DOCK);
+  gtk_window_stick (GTK_WINDOW (window));
 
+  /*
+   * Realize the window so we can get the xid.
+   */
   gtk_widget_realize (window);
 
-  g_signal_connect (window, "delete-event",
-                    G_CALLBACK (mpl_panel_gtk_window_delete_event_cb), self);
-
-  g_signal_connect (window, "notify::embedded",
-                    G_CALLBACK (mpl_panel_gtk_embedded_cb), self);
-
-  g_signal_connect (window, "size-allocate",
-                    G_CALLBACK (mpl_panel_gtk_size_allocate_cb), self);
-
   g_object_set (self, "xid", GDK_WINDOW_XID (window->window), NULL);
 }
 
@@ -196,9 +194,24 @@ mpl_panel_gtk_new (const gchar *name,
   return panel;
 }
 
+void
+mpl_panel_gtk_set_child (MplPanelGtk *panel, GtkWidget *child)
+{
+  MplPanelGtkPrivate *priv = MPL_PANEL_GTK (panel)->priv;
+
+  if (priv->child)
+    gtk_container_remove (GTK_CONTAINER (priv->window), priv->child);
+
+  priv->child = child;
+
+  gtk_container_add (GTK_CONTAINER (priv->window), child);
+}
+
 GtkWidget *
 mpl_panel_gtk_get_window (MplPanelGtk *panel)
 {
-  return panel->priv->window;
+  MplPanelGtkPrivate *priv = MPL_PANEL_GTK (panel)->priv;
+
+  return priv->window;
 }
 
