@@ -21,16 +21,23 @@
  * 02111-1307, USA.
  */
 
+#include <glib/gi18n.h>
+
 #include "mnb-drop-down.h"
+#include "mnb-panel.h"
 #include "mnb-toolbar.h"    /* For MNB_IS_TOOLBAR */
 #include "moblin-netbook.h" /* For PANEL_HEIGHT */
 #include "switcher/mnb-switcher.h"
 
 #define SLIDE_DURATION 150
 
-static void mnb_button_toggled_cb (NbtkWidget *, GParamSpec *, MnbDropDown *);
+static void mnb_panel_iface_init (MnbPanelIface *iface);
 
-G_DEFINE_TYPE (MnbDropDown, mnb_drop_down, NBTK_TYPE_TABLE)
+G_DEFINE_TYPE_WITH_CODE (MnbDropDown,
+                         mnb_drop_down,
+                         MX_TYPE_TABLE,
+                         G_IMPLEMENT_INTERFACE (MNB_TYPE_PANEL,
+                                                mnb_panel_iface_init));
 
 #define GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), MNB_TYPE_DROP_DOWN, MnbDropDownPrivate))
@@ -41,24 +48,12 @@ enum {
   PROP_MUTTER_PLUGIN,
 };
 
-enum
-{
-  SHOW_BEGIN,
-  SHOW_COMPLETED,
-  HIDE_BEGIN,
-  HIDE_COMPLETED,
-
-  LAST_SIGNAL
-};
-
-static guint dropdown_signals[LAST_SIGNAL] = { 0 };
-
 struct _MnbDropDownPrivate {
   MutterPlugin *plugin;
 
   ClutterActor *child;
   ClutterActor *footer;
-  NbtkButton *button;
+  MxButton *button;
   gint x;
   gint y;
 
@@ -66,7 +61,6 @@ struct _MnbDropDownPrivate {
 
   gboolean in_show_animation : 1;
   gboolean in_hide_animation : 1;
-  gboolean hide_toolbar      : 1;
 
   gulong show_completed_id;
   gulong hide_completed_id;
@@ -113,11 +107,15 @@ mnb_drop_down_dispose (GObject *object)
 
   if (priv->button)
     {
-      g_signal_handlers_disconnect_by_func (priv->button,
-                                            mnb_button_toggled_cb, object);
       priv->button = NULL;
     }
 
+  if (priv->footer)
+    {
+      clutter_actor_destroy (priv->footer);
+      priv->footer = NULL;
+    }
+
   G_OBJECT_CLASS (mnb_drop_down_parent_class)->dispose (object);
 }
 
@@ -133,17 +131,16 @@ mnb_drop_down_show_completed_cb (ClutterAnimation *anim, ClutterActor *actor)
   MnbDropDownPrivate *priv = MNB_DROP_DOWN (actor)->priv;
 
   priv->in_show_animation = FALSE;
-  priv->hide_toolbar = FALSE;
   priv->show_anim = NULL;
   priv->show_completed_id = 0;
 
   if (priv->button)
     {
-      if (!nbtk_button_get_checked (priv->button))
-        nbtk_button_set_checked (priv->button, TRUE);
+      if (!mx_button_get_checked (priv->button))
+        mx_button_set_checked (priv->button, TRUE);
     }
 
-  g_signal_emit (actor, dropdown_signals[SHOW_COMPLETED], 0);
+  g_signal_emit_by_name (actor, "show-completed");
   g_object_unref (actor);
 }
 
@@ -159,92 +156,6 @@ mnb_toolbar_show_completed_cb (MnbToolbar *toolbar, gpointer data)
   clutter_actor_show (dropdown);
 }
 
-/*
- * Ensure that the drop down does not stretch over any reserved strut at the
- * bottom of the screen (e.g., if VKB is present)
- */
-void
-mnb_drop_down_ensure_size (MnbDropDown *self)
-{
-  MnbDropDownPrivate *priv  = MNB_DROP_DOWN (self)->priv;
-  ClutterActor       *actor = CLUTTER_ACTOR (self);
-
-  if (priv->child)
-    {
-      MetaRectangle  r;
-      MetaScreen    *screen;
-      MetaWorkspace *workspace;
-
-      screen    = mutter_plugin_get_screen (priv->plugin);
-      workspace = meta_screen_get_active_workspace (screen);
-
-      if (workspace)
-        {
-          gfloat x, y, w, h, wc, hc;
-          gint   xi, yi, wi, hi, wci, hci;
-          gint   max_height, max_inner_height, inner_width;
-
-          meta_workspace_get_work_area_all_monitors (workspace, &r);
-
-          clutter_actor_get_position (actor, &x, &y);
-          clutter_actor_get_size (actor, &w, &h);
-          clutter_actor_get_size (priv->child, &wc, &hc);
-
-          xi  = (gint)x;
-          yi  = (gint)y;
-          wi  = (gint)w;
-          hi  = (gint)h;
-          wci = (gint)wc;
-          hci = (gint)hc;
-
-          /*
-           * Maximum height of the panel is the available working height plus the
-           * height of the panel shadow (we allow the shaddow to stretch out of
-           * the available area).
-           */
-          max_height = r.y + r.height - yi + MNB_DROP_DOWN_SHADOW_HEIGHT;
-
-          /*
-           * inner height is height of the dropdown child, i.e., the max height
-           * minus the height of the shadow, minus the height of the footer
-           * (half toolbar height), minus the y padding in the panel.
-           */
-          max_inner_height = max_height - MNB_DROP_DOWN_SHADOW_HEIGHT -
-            TOOLBAR_HEIGHT/2 - 4;
-
-          inner_width = r.width - TOOLBAR_X_PADDING * 2;
-
-          /*
-           * We have to test the size of the child here, as the external size
-           * might not be correct (e.g., when initially showing the Switcher the
-           * external size matches the size of the child, i.e., the styling is
-           * not yet applied.
-           *
-           * When initially showing the OOP panels, the child is not yet
-           * allocated, and will have size 0; in that case we force the resize
-           * otherwise the child would get allocated with the size specified at
-           * construction.
-           *
-           * We rest the size whenever the max height does not match that of the
-           * child, so that the window gets resized in both up and down (the
-           * height passed into mpl_panel_set_height() is really a maximum
-           * allowable height, and if the panel client asked for height lesser
-           * than this, the original height request will be respected.
-           */
-          if (max_inner_height != hci || inner_width != wci)
-            {
-              if (MNB_IS_PANEL (actor))
-                mnb_panel_set_size ((MnbPanel*)actor, r.width, max_height);
-              else if (MNB_IS_SWITCHER (actor))
-                mnb_switcher_set_size ((MnbSwitcher*)actor,
-                                       r.width, max_height);
-              else
-                clutter_actor_set_size (actor, w, (gfloat) max_height);
-            }
-        }
-    }
-}
-
 static void
 mnb_drop_down_show (ClutterActor *actor)
 {
@@ -268,7 +179,7 @@ mnb_drop_down_show (ClutterActor *actor)
       priv->in_hide_animation = FALSE;
     }
 
-  mnb_drop_down_ensure_size ((MnbDropDown*)actor);
+  mnb_panel_ensure_size (MNB_PANEL (actor));
 
   /*
    * Check the toolbar is visible, if not show it.
@@ -297,7 +208,7 @@ mnb_drop_down_show (ClutterActor *actor)
       return;
     }
 
-  g_signal_emit (actor, dropdown_signals[SHOW_BEGIN], 0);
+  g_signal_emit_by_name (actor, "show-begin");
 
   CLUTTER_ACTOR_CLASS (mnb_drop_down_parent_class)->show (actor);
 
@@ -342,25 +253,8 @@ mnb_drop_down_hide_completed_cb (ClutterAnimation *anim, ClutterActor *actor)
   /* now that it's hidden we can put it back to where it is suppoed to be */
   clutter_actor_set_position (actor, priv->x, priv->y);
 
-  if (priv->hide_toolbar)
-    {
-      /*
-       * If the hide_toolbar flag is set, we attempt to hide the Toolbar now
-       * that the panel is hidden.
-       */
-      ClutterActor *toolbar = clutter_actor_get_parent (actor);
-
-      while (toolbar && !MNB_IS_TOOLBAR (toolbar))
-        toolbar = clutter_actor_get_parent (toolbar);
-
-      if (toolbar)
-        mnb_toolbar_hide ((MnbToolbar*)toolbar);
-
-      priv->hide_toolbar = FALSE;
-    }
-
   priv->in_hide_animation = FALSE;
-  g_signal_emit (actor, dropdown_signals[HIDE_COMPLETED], 0);
+  g_signal_emit_by_name (actor, "hide-completed");
   g_object_unref (actor);
 }
 
@@ -384,16 +278,15 @@ mnb_drop_down_hide (ClutterActor *actor)
       priv->show_anim = NULL;
       priv->show_completed_id = 0;
       priv->in_show_animation = FALSE;
-      priv->hide_toolbar = FALSE;
 
       if (priv->button)
         {
-          if (nbtk_button_get_checked (priv->button))
-            nbtk_button_set_checked (priv->button, FALSE);
+          if (mx_button_get_checked (priv->button))
+            mx_button_set_checked (priv->button, FALSE);
         }
     }
 
-  g_signal_emit (actor, dropdown_signals[HIDE_BEGIN], 0);
+  g_signal_emit_by_name (actor, "hide-begin");
 
   /* de-activate the button */
   if (priv->button)
@@ -401,8 +294,8 @@ mnb_drop_down_hide (ClutterActor *actor)
       /* hide is hooked into the notify::checked signal from the button, so
        * make sure we don't get into a loop by checking checked first
        */
-      if (nbtk_button_get_checked (priv->button))
-        nbtk_button_set_checked (priv->button, FALSE);
+      if (mx_button_get_checked (priv->button))
+        mx_button_set_checked (priv->button, FALSE);
     }
 
   if (!priv->child)
@@ -454,26 +347,6 @@ mnb_button_event_capture (ClutterActor *actor, ClutterButtonEvent *event)
   return TRUE;
 }
 
-static void
-mnb_button_toggled_cb (NbtkWidget  *button,
-                       GParamSpec  *pspec,
-                       MnbDropDown *drop_down)
-{
-  ClutterActor *actor = CLUTTER_ACTOR (drop_down);
-
-  if (nbtk_button_get_checked (NBTK_BUTTON (button)))
-    {
-      /*
-       * Must reset the y in case a previous animation ended prematurely
-       * and the y is not set correctly; see bug 900.
-       */
-      clutter_actor_set_y (actor, TOOLBAR_HEIGHT);
-      clutter_actor_show (actor);
-    }
-  else
-    clutter_actor_hide (actor);
-}
-
 static void
 mnb_drop_down_allocate (ClutterActor          *actor,
                         const ClutterActorBox *box,
@@ -525,17 +398,19 @@ mnb_drop_down_allocate (ClutterActor          *actor,
 static void
 mnb_drop_down_constructed (GObject *object)
 {
+#if 0
   MnbDropDownPrivate *priv = MNB_DROP_DOWN (object)->priv;
-  NbtkWidget         *footer;
+  ClutterActor       *footer;
 
   /* footer with "up" button */
-  footer = nbtk_button_new ();
-  nbtk_widget_set_style_class_name (footer, "drop-down-footer");
-  nbtk_table_add_actor (NBTK_TABLE (object), CLUTTER_ACTOR (footer), 1, 0);
+  footer = mx_button_new ();
+  mx_stylable_set_style_class (MX_STYLABLE (footer), "drop-down-footer");
+  mx_table_add_actor (MX_TABLE (object), CLUTTER_ACTOR (footer), 1, 0);
   g_signal_connect_swapped (footer, "clicked",
-                            G_CALLBACK (mnb_drop_down_hide_with_toolbar), object);
+                            G_CALLBACK (mnb_panel_hide_with_toolbar), object);
 
   priv->footer = CLUTTER_ACTOR (footer);
+#endif
 }
 
 static void
@@ -548,9 +423,9 @@ mnb_drop_down_class_init (MnbDropDownClass *klass)
 
   object_class->get_property = mnb_drop_down_get_property;
   object_class->set_property = mnb_drop_down_set_property;
-  object_class->dispose = mnb_drop_down_dispose;
-  object_class->finalize = mnb_drop_down_finalize;
-  object_class->constructed = mnb_drop_down_constructed;
+  object_class->dispose      = mnb_drop_down_dispose;
+  object_class->finalize     = mnb_drop_down_finalize;
+  object_class->constructed  = mnb_drop_down_constructed;
 
   clutter_class->show = mnb_drop_down_show;
   clutter_class->hide = mnb_drop_down_hide;
@@ -560,42 +435,6 @@ mnb_drop_down_class_init (MnbDropDownClass *klass)
   clutter_class->button_release_event = mnb_button_event_capture;
   clutter_class->allocate = mnb_drop_down_allocate;
 
-  dropdown_signals[SHOW_BEGIN] =
-    g_signal_new ("show-begin",
-                  G_TYPE_FROM_CLASS (object_class),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (MnbDropDownClass, show_begin),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
-
-  dropdown_signals[SHOW_COMPLETED] =
-    g_signal_new ("show-completed",
-                  G_TYPE_FROM_CLASS (object_class),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (MnbDropDownClass, show_completed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
-
-  dropdown_signals[HIDE_BEGIN] =
-    g_signal_new ("hide-begin",
-                  G_TYPE_FROM_CLASS (object_class),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (MnbDropDownClass, hide_begin),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
-
-  dropdown_signals[HIDE_COMPLETED] =
-    g_signal_new ("hide-completed",
-                  G_TYPE_FROM_CLASS (object_class),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (MnbDropDownClass, hide_completed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
-
   g_object_class_install_property (object_class,
                                    PROP_MUTTER_PLUGIN,
                                    g_param_spec_object ("mutter-plugin",
@@ -617,7 +456,7 @@ mnb_drop_down_init (MnbDropDown *self)
                 NULL);
 }
 
-NbtkWidget*
+MxWidget*
 mnb_drop_down_new (MutterPlugin *plugin)
 {
   return g_object_new (MNB_TYPE_DROP_DOWN,
@@ -673,7 +512,7 @@ mnb_drop_down_set_child (MnbDropDown *drop_down,
         g_signal_connect (child, "parent-set",
                           G_CALLBACK (mnb_drop_down_reparent_cb),
                           drop_down);
-      nbtk_table_add_actor (NBTK_TABLE (drop_down), child, 0, 0);
+      mx_table_add_actor (MX_TABLE (drop_down), child, 0, 0);
     }
 
   priv->child = child;
@@ -693,14 +532,14 @@ mnb_drop_down_button_weak_unref_cb (MnbDropDown *drop_down, GObject *button)
   drop_down->priv->button = NULL;
 }
 
-void
-mnb_drop_down_set_button (MnbDropDown *drop_down,
-                          NbtkButton *button)
+static void
+mnb_drop_down_panel_set_button (MnbPanel *panel, MxButton *button)
 {
-  NbtkButton *old_button;
+  MnbDropDown *drop_down = MNB_DROP_DOWN (panel);
+  MxButton  *old_button;
 
   g_return_if_fail (MNB_IS_DROP_DOWN (drop_down));
-  g_return_if_fail (!button || NBTK_IS_BUTTON (button));
+  g_return_if_fail (!button || MX_IS_BUTTON (button));
 
   old_button = drop_down->priv->button;
   drop_down->priv->button = button;
@@ -710,10 +549,6 @@ mnb_drop_down_set_button (MnbDropDown *drop_down,
       g_object_weak_unref (G_OBJECT (old_button),
                            (GWeakNotify) mnb_drop_down_button_weak_unref_cb,
                            drop_down);
-
-      g_signal_handlers_disconnect_by_func (old_button,
-                                            G_CALLBACK (mnb_button_toggled_cb),
-                                            drop_down);
     }
 
   if (button)
@@ -721,31 +556,9 @@ mnb_drop_down_set_button (MnbDropDown *drop_down,
       g_object_weak_ref (G_OBJECT (button),
                          (GWeakNotify) mnb_drop_down_button_weak_unref_cb,
                          drop_down);
-
-      g_signal_connect (button,
-                        "notify::checked",
-                        G_CALLBACK (mnb_button_toggled_cb),
-                        drop_down);
     }
 }
 
-/*
- * Hides both dropdown and toolbar, in a sequnce so as to preserve the
- * hide animations.
- */
-void
-mnb_drop_down_hide_with_toolbar (MnbDropDown *self)
-{
-  MnbDropDownPrivate *priv = self->priv;
-
-  priv->hide_toolbar = TRUE;
-
-  if (priv->in_hide_animation)
-    return;
-
-  clutter_actor_hide (CLUTTER_ACTOR (self));
-}
-
 /*
  * Returns untransformed geometry of the footer relative to the drop down.
  */
@@ -768,6 +581,95 @@ mnb_drop_down_get_footer_geometry (MnbDropDown *self,
   *x      = clutter_actor_get_x (CLUTTER_ACTOR (self));
   *y      = clutter_actor_get_height (priv->child);
   *width  = clutter_actor_get_width  (CLUTTER_ACTOR (self));
-  *height = clutter_actor_get_height (priv->footer);
+  *height = /*clutter_actor_get_height (priv->footer)*/ 0;
+}
+
+static void
+mnb_drop_down_panel_show (MnbPanel *panel)
+{
+  clutter_actor_show (CLUTTER_ACTOR (panel));
+}
+
+static void
+mnb_drop_down_panel_hide (MnbPanel *panel)
+{
+  clutter_actor_hide (CLUTTER_ACTOR (panel));
+}
+
+static void
+mnb_drop_down_panel_set_size (MnbPanel *panel, guint width, guint height)
+{
+  clutter_actor_set_size (CLUTTER_ACTOR (panel), width, height);
+}
+
+static void
+mnb_drop_down_panel_get_size (MnbPanel *panel, guint *width, guint *height)
+{
+  MnbDropDownPrivate *priv = MNB_DROP_DOWN (panel)->priv;
+  gfloat w, h;
+  gfloat wc = 0.0, hc = 0.0;
+
+  /*
+   * FIXME -- need to find out why the height of the panel is reported as
+   * 0 and fix that; this is a very ugly cludge.
+   */
+  clutter_actor_get_size (CLUTTER_ACTOR (panel), &w, &h);
+
+  if (priv->child)
+    clutter_actor_get_size (priv->child, &wc, &hc);
+
+  if (width)
+    {
+      if (w)
+        *width = w;
+      else if (wc)
+        *width = wc + TOOLBAR_X_PADDING * 2;
+      else
+        *width = 0;
+    }
+
+  if (height)
+    {
+      if (h)
+        *height = h;
+      else if (hc)
+        *height = hc + TOOLBAR_HEIGHT / 2 + 4;
+      else
+        *height = 0;
+    }
+}
+
+static void
+mnb_drop_down_panel_set_position (MnbPanel *panel, gint x, gint y)
+{
+  clutter_actor_set_position (CLUTTER_ACTOR (panel), x, y);
+}
+
+static void
+mnb_drop_down_panel_get_position (MnbPanel *panel, gint *x, gint *y)
+{
+  gfloat xf, yf;
+
+  clutter_actor_get_position (CLUTTER_ACTOR (panel), &xf, &yf);
+
+  if (x)
+    *x = xf;
+
+  if (y)
+    *y = yf;
+}
+
+static void
+mnb_panel_iface_init (MnbPanelIface *iface)
+{
+  iface->show             = mnb_drop_down_panel_show;
+  iface->hide             = mnb_drop_down_panel_hide;
+
+  iface->set_size         = mnb_drop_down_panel_set_size;
+  iface->get_size         = mnb_drop_down_panel_get_size;
+  iface->set_position     = mnb_drop_down_panel_set_position;
+  iface->get_position     = mnb_drop_down_panel_get_position;
+
+  iface->set_button       = mnb_drop_down_panel_set_button;
 }
 
