@@ -26,7 +26,7 @@
 
 #include <glib-object.h>
 #include <clutter/clutter.h>
-#include <nbtk/nbtk.h>
+#include <mx/mx.h>
 #include "moblin-netbook.h"
 
 /*
@@ -60,40 +60,29 @@ G_BEGIN_DECLS
 typedef struct _MnbDropDownPrivate MnbDropDownPrivate;
 
 typedef struct {
-  NbtkTable parent;
+  MxTable parent;
   /*< private >*/
   MnbDropDownPrivate *priv;
 } MnbDropDown;
 
 typedef struct {
-  NbtkTableClass parent_class;
+  MxTableClass parent_class;
 
-  void (*show_begin)     (MnbDropDown *drop_down);
-  void (*show_completed) (MnbDropDown *drop_down);
-  void (*hide_begin)     (MnbDropDown *drop_down);
-  void (*hide_completed) (MnbDropDown *drop_down);
 } MnbDropDownClass;
 
 GType mnb_drop_down_get_type (void);
 
-NbtkWidget*   mnb_drop_down_new (MutterPlugin *plugin);
+MxWidget*     mnb_drop_down_new (MutterPlugin *plugin);
 
 void          mnb_drop_down_set_child (MnbDropDown *drop_down, ClutterActor *child);
 ClutterActor* mnb_drop_down_get_child (MnbDropDown *drop_down);
 
-
-void          mnb_drop_down_set_button (MnbDropDown *drop_down, NbtkButton *button);
-
-void          mnb_drop_down_hide_with_toolbar (MnbDropDown *drop_down);
-
 void          mnb_drop_down_get_footer_geometry (MnbDropDown *drop_down,
                                                  gfloat      *x,
                                                  gfloat      *y,
                                                  gfloat      *width,
                                                  gfloat      *height);
 
-void          mnb_drop_down_ensure_size (MnbDropDown *self);
-
 G_END_DECLS
 
 #endif /* _MNB_DROP_DOWN */
