@@ -1,7 +1,7 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 
 /*
- * Copyright (c) 2008 Intel Corp.
+ * Copyright (c) 2008, 2010 Intel Corp.
  *
  * Author: Tomas Frydrych <tf@linux.intel.com>
  *         Thomas Wood <thomas@linux.intel.com>
@@ -32,6 +32,9 @@
 #include "mnb-toolbar.h"
 #include "effects/mnb-switch-zones-effect.h"
 #include "notifications/mnb-notification-gtk.h"
+#include "mnb-panel-frame.h"
+#include "moblin-netbook-constraints.h"
+#include "moblin-netbook-mutter-hints.h"
 
 #include <compositor-mutter.h>
 #include <display.h>
@@ -56,6 +59,7 @@
 #include <string.h>
 #include <signal.h>
 #include <X11/extensions/Xcomposite.h>
+#include <X11/extensions/Xrandr.h>
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 
@@ -67,8 +71,10 @@
 #define MYZONE_TIMEOUT              200
 #define ACTOR_DATA_KEY "MCCP-moblin-netbook-actor-data"
 #define NOTIFICATION_KEY "MNB-MW-urgent-notification"
-#define KEY_DIR "/desktop/moblin/background"
-#define KEY_BG_FILENAME KEY_DIR "/picture_filename"
+#define BG_KEY_DIR "/desktop/moblin/background"
+#define KEY_BG_FILENAME BG_KEY_DIR "/picture_filename"
+#define THEME_KEY_DIR "/apps/metacity/general"
+#define KEY_THEME THEME_KEY_DIR "/theme"
 
 static MutterPlugin *plugin_singleton = NULL;
 
@@ -268,9 +274,9 @@ on_urgent_notify_allocation_cb (ClutterActor *notify_urgent,
 }
 
 static void
-moblin_netbook_workarea_changed_foreach (MnbDropDown *panel, gpointer data)
+moblin_netbook_workarea_changed_foreach (MnbPanel *panel, gpointer data)
 {
-  mnb_drop_down_ensure_size (panel);
+  mnb_panel_ensure_size (panel);
 }
 
 /*
@@ -360,6 +366,51 @@ moblin_netbook_workspace_switched_cb (MetaScreen          *screen,
   moblin_netbook_toggle_compositor (plugin, on);
 }
 
+static void
+moblin_netbook_compute_screen_size (Display *xdpy,
+                                    gint     screen_no,
+                                    gint    *width_mm,
+                                    gint    *height_mm)
+{
+  Window                  xroot = RootWindow (xdpy, screen_no);
+  XRRScreenSize          *sizes;
+  XRRScreenConfiguration *cfg = NULL;
+  SizeID                  current;
+  Rotation                rotation;
+  gint                    n_sizes;
+
+  cfg = XRRGetScreenInfo (xdpy, xroot);
+
+  if (!cfg)
+    goto fallback;
+
+  current = XRRConfigCurrentConfiguration (cfg, &rotation);
+
+  sizes = XRRConfigSizes (cfg, &n_sizes);
+
+  if (current < 0 || !sizes)
+    goto fallback;
+
+  *width_mm  = sizes[current].mwidth;
+  *height_mm = sizes[current].mheight;
+
+  XRRFreeScreenConfigInfo (cfg);
+  return;
+
+ fallback:
+  if (cfg)
+    XRRFreeScreenConfigInfo (cfg);
+
+  g_warning ("Could not retrieve screen info via xrandr");
+
+  /*
+   * Fall back on the server info; this is so inaccurate that it is
+   * useless.
+   */
+  *width_mm  = XDisplayWidthMM  (xdpy, screen_no);
+  *height_mm = XDisplayHeightMM (xdpy, screen_no);
+}
+
 static void
 moblin_netbook_plugin_constructed (GObject *object)
 {
@@ -379,19 +430,65 @@ moblin_netbook_plugin_constructed (GObject *object)
   ClutterColor   low_clr = { 0, 0, 0, 0x7f };
   GError        *err = NULL;
 
-  MetaScreen   *screen  = mutter_plugin_get_screen (MUTTER_PLUGIN (plugin));
-  MetaDisplay  *display = meta_screen_get_display (screen);
-  ClutterActor *stage   = mutter_get_stage_for_screen (screen);
+  MetaScreen   *screen    = mutter_plugin_get_screen (MUTTER_PLUGIN (plugin));
+  MetaDisplay  *display   = meta_screen_get_display (screen);
+  ClutterActor *stage     = mutter_get_stage_for_screen (screen);
+  Display      *xdpy      = meta_display_get_xdisplay (display);
+  guint         screen_no = meta_screen_get_screen_number (screen);
+
+  gint          screen_width_mm  = XDisplayWidthMM (xdpy, screen_no);
+  gint          screen_height_mm = XDisplayHeightMM (xdpy, screen_no);
+  gboolean      netbook_mode = FALSE;
+  GConfClient  *gconf_client;
+
+  moblin_netbook_compute_screen_size (xdpy,
+                                      screen_no,
+                                      &screen_width_mm,
+                                      &screen_height_mm);
+
+  g_debug ("Screen size %dmm x %dmm", screen_width_mm, screen_height_mm);
+
+  if (screen_width_mm < 280)
+    netbook_mode = priv->netbook_mode = TRUE;
 
   plugin_singleton = (MutterPlugin*)object;
 
+  {
+    gchar *moblin_session;
+    Atom   atom__MOBLIN;
+
+    atom__MOBLIN = XInternAtom (xdpy, "_MOBLIN", False);
+
+    moblin_session =
+      g_strdup_printf ("session-type=%s:frame-style=naked",
+                       netbook_mode ? "netbook" : "nettop");
+
+    g_debug ("Setting _MOBLIN=%s", moblin_session);
+
+    meta_error_trap_push (display);
+    XChangeProperty (xdpy,
+                     meta_screen_get_xroot (screen),
+                     atom__MOBLIN,
+                     XA_STRING,
+                     8, PropModeReplace,
+                     (unsigned char*)moblin_session, strlen (moblin_session));
+    meta_error_trap_pop (display, FALSE);
+
+    g_free (moblin_session);
+  }
+
+  gconf_client = priv->gconf_client = gconf_client_get_default ();
+
+  if (!netbook_mode)
+    gconf_client_set_string (gconf_client, KEY_THEME, "Moblin-Nettop", NULL);
+
   /* tweak with env var as then possible to develop in desktop env. */
   if (!g_getenv("MUTTER_DISABLE_WS_CLAMP"))
     meta_prefs_set_num_workspaces (1);
 
-  nbtk_texture_cache_load_cache (nbtk_texture_cache_get_default (),
-                                 THEMEDIR "/nbtk.cache");
-  nbtk_style_load_from_file (nbtk_style_get_default (),
+  mx_texture_cache_load_cache (mx_texture_cache_get_default (),
+                                 THEMEDIR "/mx.cache");
+  mx_style_load_from_file (mx_style_get_default (),
                              THEMEDIR "/mutter-moblin.css",
                              &err);
   if (err)
@@ -451,23 +548,18 @@ moblin_netbook_plugin_constructed (GObject *object)
 
   mnb_input_manager_create (MUTTER_PLUGIN (plugin));
 
+  setup_focus_window (MUTTER_PLUGIN (plugin));
+
   /*
    * This also creates the launcher.
    */
   toolbar = priv->toolbar =
     CLUTTER_ACTOR (mnb_toolbar_new (MUTTER_PLUGIN (plugin)));
 
-#if 1
-  mnb_toolbar_append_panel_old (MNB_TOOLBAR (toolbar),
-                                MPL_PANEL_ZONES, _("zones"));
-#endif
-
   clutter_set_motion_events_enabled (TRUE);
 
   desktop_background_init (MUTTER_PLUGIN (plugin));
 
-  setup_focus_window (MUTTER_PLUGIN (plugin));
-
   /* Notifications */
   priv->notify_store = moblin_netbook_notify_store_new ();
 
@@ -540,6 +632,50 @@ moblin_netbook_plugin_constructed (GObject *object)
   meta_prefs_override_no_tab_popup (TRUE);
 }
 
+static void
+moblin_netbook_panel_frame_footer_cb (MnbPanelFrame *frame, MutterWindow *mcw)
+{
+  MutterPlugin               *plugin  = moblin_netbook_get_plugin_singleton ();
+  MoblinNetbookPluginPrivate *priv    = MOBLIN_NETBOOK_PLUGIN (plugin)->priv;
+  Window                      xwin    = mutter_window_get_x_window (mcw);
+  MnbToolbar                 *toolbar = MNB_TOOLBAR (priv->toolbar);
+  MnbPanel                   *panel;
+
+  if ((panel = mnb_toolbar_find_panel_for_xid (toolbar, xwin)))
+    {
+      mnb_panel_hide_with_toolbar (panel);
+    }
+}
+
+static MutterShadow *
+moblin_netbook_get_shadow (MutterPlugin *plugin, MutterWindow *mcw)
+{
+  MetaCompWindowType type;
+
+  type = mutter_window_get_window_type (mcw);
+
+  if (type == META_COMP_WINDOW_DOCK)
+    {
+      MutterShadow *shadow = mutter_shadow_new ();
+
+      shadow->actor = (ClutterActor*) mnb_panel_frame_new ();
+
+      g_signal_connect (shadow->actor,
+                        "footer-clicked",
+                        G_CALLBACK (moblin_netbook_panel_frame_footer_cb),
+                        mcw);
+
+      shadow->attach_left   = -4;
+      shadow->attach_top    = -4;
+      shadow->attach_right  =  4;
+      shadow->attach_bottom = 12;
+
+      return shadow;
+    }
+
+  return NULL;
+}
+
 static void
 moblin_netbook_plugin_class_init (MoblinNetbookPluginClass *klass)
 {
@@ -561,6 +697,8 @@ moblin_netbook_plugin_class_init (MoblinNetbookPluginClass *klass)
   plugin_class->kill_effect      = kill_effect;
   plugin_class->plugin_info      = plugin_info;
   plugin_class->xevent_filter    = xevent_filter;
+  plugin_class->get_shadow       = moblin_netbook_get_shadow;
+  plugin_class->constrain_window = moblin_netbook_constrain_window;
 
   g_type_class_add_private (gobject_class, sizeof (MoblinNetbookPluginPrivate));
 }
@@ -964,7 +1102,8 @@ maybe_show_myzone (MutterPlugin *plugin)
     }
 
   if (no_apps)
-    mnb_toolbar_activate_panel (MNB_TOOLBAR (priv->toolbar), MPL_PANEL_MYZONE);
+    mnb_toolbar_activate_panel (MNB_TOOLBAR (priv->toolbar),
+                                "moblin-panel-myzone");
 
   return FALSE;
 }
@@ -1064,15 +1203,15 @@ check_for_empty_workspace (MutterPlugin *plugin,
         }
 
       meta_screen_remove_workspace (screen, current_ws, timestamp);
-    }
 
-  /*
-   * Check if we show the myzone; we do this after a short timeout, so that
-   * we do not drop the myzone if the application does something like first
-   * pop a root password dialog, and only after the dialog closes, creates its
-   * main window (see MB#4766).
-   */
-  g_timeout_add (MYZONE_TIMEOUT, (GSourceFunc)maybe_show_myzone, plugin);
+      /*
+       * Check if we show the myzone; we do this after a short timeout, so that
+       * we do not drop the myzone if the application does something like first
+       * pop a root password dialog, and only after the dialog closes, creates
+       * its main window (see MB#4766).
+       */
+      g_timeout_add (MYZONE_TIMEOUT, (GSourceFunc)maybe_show_myzone, plugin);
+    }
 }
 
 /*
@@ -1133,6 +1272,21 @@ handle_window_destruction (MutterWindow *mcw, MutterPlugin *plugin)
   wm_class  = meta_window_get_wm_class (meta_win);
   wm_name   = meta_window_get_title (meta_win);
 
+  /*
+   * Remove the window from the fullscreen list; do this unconditionally,
+   * so that under no circumstance we leave a dangling pointer behind.
+   */
+  fullscreen_app_removed (plugin, meta_win);
+
+  /*
+   * Disconnect the fullscreen notification handler; strictly speaking
+   * this should not be necessary, as the MetaWindow should be going away,
+   * but take no chances.
+   */
+  g_signal_handlers_disconnect_by_func (meta_win,
+                                        meta_window_fullscreen_notify_cb,
+                                        plugin);
+
   if (type == META_COMP_WINDOW_NORMAL)
     {
       g_signal_handlers_disconnect_by_func (mcw,
@@ -1153,21 +1307,6 @@ handle_window_destruction (MutterWindow *mcw, MutterPlugin *plugin)
           if (pid)
             kill (pid, SIGKILL);
         }
-
-      /*
-       * Remove the window from the fullscreen list; do this unconditionally,
-       * so that under no circumstance we leave a dangling pointer behind.
-       */
-      fullscreen_app_removed (plugin, meta_win);
-
-      /*
-       * Disconnect the fullscreen notification handler; strictly speaking
-       * this should not be necessary, as the MetaWindow should be going away,
-       * but take no chances.
-       */
-      g_signal_handlers_disconnect_by_func (meta_win,
-                                            meta_window_fullscreen_notify_cb,
-                                            plugin);
     }
 
   /*
@@ -1181,8 +1320,10 @@ handle_window_destruction (MutterWindow *mcw, MutterPlugin *plugin)
    * NB: This must come before we notify Mutter that the effect completed,
    *     otherwise the destruction of this window will be completed and the
    *     workspace switch effect will crash.
+   */
 
- */ if (type != META_COMP_WINDOW_SPLASHSCREEN &&
+  if ((type == META_COMP_WINDOW_NORMAL ||
+       type == META_COMP_WINDOW_DIALOG ) &&
       !mnb_toolbar_owns_window ((MnbToolbar*)priv->toolbar, mcw))
     check_for_empty_workspace (plugin, workspace, meta_win, TRUE);
 }
@@ -1206,6 +1347,7 @@ meta_window_workspace_changed_cb (MetaWindow *mw,
 {
   MutterPlugin *plugin = MUTTER_PLUGIN (data);
 
+#if 0
   /*
    * Flush any pending changes to the visibility of the window.
    * (bug 1008 suggests that the removal of an empty workspace is sometimes
@@ -1222,6 +1364,7 @@ meta_window_workspace_changed_cb (MetaWindow *mw,
    * by flushing the state in the intermediate stage for the alpha2 release.
    */
   meta_window_calc_showing (mw);
+#endif
 
   check_for_empty_workspace (plugin, old_workspace, mw, FALSE);
 }
@@ -1350,181 +1493,6 @@ meta_window_fullscreen_notify_cb (GObject    *object,
     fullscreen_app_removed (data, mw);
 }
 
-
-/*
- * SCIM functionality
- *
- * The basic problem is that the IM is implemented using X windows for the
- * preview, toolbar, and various other UI elements; this is fine for regular
- * applications, but does not work for UX Shell UI (the UI is always above
- * the windows, so, the IM windows are covered and non-interactive.
- *
- * To work around this, when a window of WM_CLASS Scim-bridge-gtk maps, we
- * create a clone and place it to the top of the stage, and we also punch a
- * hole into the stage input region so the window can get events.
- */
-
-/*
- * When the real window moves, move the clone.
- */
-static void
-scim_preview_allocation_cb (ClutterActor *source,
-                            GParamSpec   *pspec,
-                            ClutterActor *clone)
-{
-  gfloat x, y, w, h;
-
-  clutter_actor_get_position (source, &x, &y);
-  clutter_actor_get_size (source, &w, &h);
-
-  clutter_actor_set_position (clone, x, y);
-  clutter_actor_set_size (clone, w, h);
-}
-
-/*
- * When the original window is destroyed, destroy the clone.
- */
-static void
-scim_preview_destroy_cb (ClutterActor *source, ClutterActor *clone)
-{
-  ClutterActor *parent = clutter_actor_get_parent (clone);
-
-  if (parent)
-    clutter_container_remove_actor (CLUTTER_CONTAINER (parent), clone);
-  else
-    clutter_actor_destroy (clone);
-}
-
-static void
-scim_preview_raised_cb (MetaWindow *mw, ClutterActor *clone)
-{
-  clutter_actor_raise_top (clone);
-}
-
-/*
- * The following functions are trying to catch out possible corner cases where
- * our clone is left unpainted. Unfortunately, this still happens.
- */
-
-/*
- * When the inner texture alloation changes, make sure to queue redraw of the
- * clone. (Should be unnecessary.)
- */
-static void
-scim_preview_texture_allocation_cb (ClutterActor *source,
-                                    GParamSpec   *pspec,
-                                    ClutterActor *clone)
-{
-  clutter_actor_queue_redraw (clone);
-}
-
-/*
- * Make sure that any queued redraws on the inner texture get propagated to
- * our clone. (Should be unnecessary.)
- */
-static void
-scim_preview_queue_redraw_cb (ClutterActor *source,
-                              ClutterActor *origin,
-                              ClutterActor *clone)
-{
-  clutter_actor_queue_redraw (clone);
-}
-
-/*
- * Because of the way the live windows are implemented, hiding of MutterWindows
- * is accomplished by moving them into a special hidden group; so we have to
- * watch the parent-set signal, and if the parent matches the window group, we
- * make the actor visible, otherwise we hide it.
- *
- * (Should be unnecessary; the IM windows are not getting hidden from within
- * the compositor but from the IM control process, and as such undergo real
- * unmap.)
- */
-static void
-scim_preview_parent_set_cb (ClutterActor *source,
-                            ClutterActor *old_parent,
-                            ClutterActor *clone)
-{
-  MutterPlugin *plugin = moblin_netbook_get_plugin_singleton ();
-  MetaScreen   *screen = mutter_plugin_get_screen (plugin);
-  ClutterActor *wgroup = mutter_get_window_group_for_screen (screen);
-  ClutterActor *parent = clutter_actor_get_parent (source);
-
-  if (parent != wgroup)
-    clutter_actor_hide (clone);
-  else
-    {
-      clutter_actor_show (clone);
-      clutter_actor_raise_top (clone);
-    }
-}
-
-/* missing prototype; for some reason frame.h is not installed. */
-Window meta_frame_get_xwindow (MetaFrame *frame);
-
-static void
-handle_panel_child (MutterPlugin *plugin, MutterWindow *mcw)
-{
-  MetaWindow   *mw      = mutter_window_get_meta_window (mcw);
-  ClutterActor *clone   = clutter_clone_new (CLUTTER_ACTOR(mcw));
-  MetaScreen   *screen  = mutter_plugin_get_screen (plugin);
-  MetaDisplay  *display = meta_screen_get_display (screen);
-  ClutterActor *stage   = mutter_get_stage_for_screen (screen);
-  gfloat        x, y;
-  Window        xid;
-
-  /*
-   * Make a clone and place it on the top of stage.
-   */
-  clutter_actor_get_position (CLUTTER_ACTOR (mcw), &x, &y);
-  clutter_actor_set_position (clone, x, y);
-
-  clutter_container_add_actor (CLUTTER_CONTAINER (stage),
-                               clone);
-  clutter_actor_raise_top (clone);
-
-  g_signal_connect (mutter_window_get_texture (mcw),
-                    "notify::allocation",
-                    G_CALLBACK (scim_preview_texture_allocation_cb),
-                    clone);
-
-  g_signal_connect (mw,
-                    "raised",
-                    G_CALLBACK (scim_preview_raised_cb),
-                    clone);
-
-  g_signal_connect (mcw, "notify::allocation",
-                    G_CALLBACK (scim_preview_allocation_cb),
-                    clone);
-
-  g_signal_connect (mcw, "destroy",
-                    G_CALLBACK (scim_preview_destroy_cb),
-                    clone);
-
-  g_signal_connect (mutter_window_get_texture (mcw),
-                    "queue-redraw",
-                    G_CALLBACK (scim_preview_queue_redraw_cb),
-                    clone);
-
-  g_signal_connect (mcw, "parent-set",
-                    G_CALLBACK (scim_preview_parent_set_cb),
-                    clone);
-
-  mnb_input_manager_push_window (mcw,
-                                 MNB_INPUT_LAYER_PANEL_TRANSIENTS);
-
-  if (meta_window_get_frame (mw))
-    xid = meta_frame_get_xwindow (meta_window_get_frame (mw));
-  else
-    xid = meta_window_get_xwindow (mw);
-
-  meta_error_trap_push (display);
-
-  XRaiseWindow (GDK_DISPLAY (), xid);
-
-  meta_error_trap_pop (display, TRUE);
-}
-
 /*
  * Returns TRUE if the panel should close when this window appears.
  */
@@ -1560,6 +1528,40 @@ meta_window_is_blessed_window (MetaWindow *mw)
   return FALSE;
 }
 
+static gboolean
+moblin_netbook_window_is_modal_for_panel (MnbPanelOop *panel,
+                                          MetaWindow  *window)
+{
+  MutterWindow *mcw;
+  MetaWindow   *mw;
+  MetaWindow   *trans_for;
+
+  /*
+   * meta_window_get_transient_for() is quite involved, so only call it if
+   * we need to.
+   */
+  if (!meta_window_is_modal (window) ||
+      !(trans_for = meta_window_get_transient_for (window)))
+    {
+      return FALSE;
+    }
+
+  mcw = mnb_panel_oop_get_mutter_window (panel);
+  mw  = mutter_window_get_meta_window (mcw);
+
+  if (trans_for == mw)
+    return TRUE;
+
+  return FALSE;
+}
+
+static void
+moblin_netbook_panel_modal_destroyed_cb (MutterWindow *mcw,
+                                         MnbPanelOop  *panel)
+{
+  mnb_panel_oop_set_auto_modal (panel, FALSE);
+}
+
 /*
  * Simple map handler: it applies a scale effect which must be reversed on
  * completion).
@@ -1571,62 +1573,65 @@ map (MutterPlugin *plugin, MutterWindow *mcw)
   MnbToolbar                 *toolbar = MNB_TOOLBAR (priv->toolbar);
   ClutterActor               *actor = CLUTTER_ACTOR (mcw);
   MetaCompWindowType          type;
-  MnbPanel                   *active_panel;
+  MnbPanelOop                *active_panel;
   Window                      xwin;
+  MetaWindow                 *mw;
+  gboolean                    fullscreen = FALSE;
+  gboolean                    move_window = FALSE;
 
-  active_panel = (MnbPanel*)mnb_toolbar_get_active_panel (toolbar);
+  active_panel = (MnbPanelOop*) mnb_toolbar_get_active_panel (toolbar);
   type         = mutter_window_get_window_type (mcw);
   xwin         = mutter_window_get_x_window (mcw);
+  mw           = mutter_window_get_meta_window (mcw);
 
-  if (active_panel && MNB_IS_PANEL (active_panel) &&
-      mnb_panel_owns_window (active_panel, mcw))
+  if (active_panel &&
+      moblin_netbook_window_is_modal_for_panel (active_panel, mw))
     {
-      mutter_plugin_effect_completed (plugin, mcw,
-                                      MUTTER_PLUGIN_MAP);
-
-      handle_panel_child (plugin, mcw);
-      return;
+      mnb_panel_oop_set_auto_modal ((MnbPanelOop*)active_panel, TRUE);
+      g_signal_connect (mcw, "window-destroyed",
+                        G_CALLBACK (moblin_netbook_panel_modal_destroyed_cb),
+                        active_panel);
     }
 
+  g_object_get (mw, "fullscreen", &fullscreen, NULL);
+
+  if (fullscreen)
+    {
+      MetaWorkspace *ws = meta_window_get_workspace (mw);
+
+      if (ws)
+        {
+          fullscreen_app_added (plugin, mw);
+          clutter_actor_hide (CLUTTER_ACTOR (mcw));
+        }
+    }
+
+  g_signal_connect (mw, "notify::fullscreen",
+                    G_CALLBACK (meta_window_fullscreen_notify_cb),
+                    plugin);
+
   /*
    * The OR test must come before the type test since GTK_WINDOW_POPUP type
    * windows are both override redirect, but also have a _NET_WM_WINDOW_TYPE set
    * to NORMAL
    */
-  else if (mutter_window_is_override_redirect (mcw))
+  if (mutter_window_is_override_redirect (mcw))
     {
-      MetaWindow  *mw       = mutter_window_get_meta_window (mcw);
       const gchar *wm_class = meta_window_get_wm_class (mw);
-      MnbPanel    *panel;
+
+      mutter_plugin_effect_completed (plugin, mcw, MUTTER_PLUGIN_MAP);
 
       /*
-       * Handle the case of a IM preview window for a panel.
-       *
-       * We want to narrow this down before calling XGetProperty().
+       * If this is a SCIM window, ensure we have an input region for it on the
+       * top of the stack (so that events get through even if a panel is
+       * present).
        */
       if (wm_class && !strcmp (wm_class, "Scim-panel-gtk"))
-        {
-          if (active_panel)
-            {
-              /*
-               * Let the compositor to finish up mapping of this window.
-               */
-              mutter_plugin_effect_completed (plugin, mcw,
-                                              MUTTER_PLUGIN_MAP);
-
-
-              handle_panel_child (plugin, mcw);
-              return;
-            }
-
-          /*
-           * We know by now that this window requires just a normal OR
-           * treatment, so exit here to avoid testing it against panels and
-           * system tray windows.
-           */
-          mutter_plugin_effect_completed (plugin, mcw, MUTTER_PLUGIN_MAP);
-          return;
-        }
+        mnb_input_manager_push_window (mcw, MNB_INPUT_LAYER_TOP);
+    }
+  else if (type == META_COMP_WINDOW_DOCK)
+    {
+      MnbPanel *panel;
 
       if ((panel = mnb_toolbar_find_panel_for_xid (toolbar, xwin)))
         {
@@ -1636,15 +1641,11 @@ map (MutterPlugin *plugin, MutterWindow *mcw)
            * effect completes, we have to first let the compositor to do its
            * thing, and only then impose our will on it.
            */
+          g_object_set (mcw, "shadow-type", MUTTER_SHADOW_ALWAYS, NULL);
           mutter_plugin_effect_completed (plugin, mcw, MUTTER_PLUGIN_MAP);
-          mnb_panel_show_mutter_window (panel, mcw);
+          mnb_panel_oop_show_mutter_window (MNB_PANEL_OOP (panel), mcw);
         }
-      else
-        mutter_plugin_effect_completed (plugin, mcw, MUTTER_PLUGIN_MAP);
-
     }
-
-
   /*
    * Anything that might be associated with startup notification needs to be
    * handled here; if this list grows, we should just split it further.
@@ -1656,35 +1657,13 @@ map (MutterPlugin *plugin, MutterWindow *mcw)
     {
       ClutterAnimation   *animation;
       ActorPrivate       *apriv = get_actor_private (mcw);
-      MetaWindow         *mw    = mutter_window_get_meta_window (mcw);
       MetaScreen        *screen = mutter_plugin_get_screen (plugin);
       gint               screen_width, screen_height;
       gfloat             actor_width, actor_height;
-      gboolean           fullscreen = FALSE;
 
-      if (mw)
-        {
-          g_object_get (mw, "fullscreen", &fullscreen, NULL);
-
-          if (fullscreen)
-            {
-              MetaWorkspace *ws = meta_window_get_workspace (mw);
-
-              if (ws)
-                {
-                  fullscreen_app_added (plugin, mw);
-                  clutter_actor_hide (CLUTTER_ACTOR (mcw));
-                }
-            }
-
-          g_signal_connect (mw, "notify::fullscreen",
-                            G_CALLBACK (meta_window_fullscreen_notify_cb),
-                            plugin);
-
-          /* Hide toolbar etc in presence of modal window */
-          if (meta_window_is_blessed_window (mw))
-            mnb_toolbar_hide (MNB_TOOLBAR (priv->toolbar));
-        }
+      /* Hide toolbar etc in presence of modal window */
+      if (meta_window_is_blessed_window (mw))
+        mnb_toolbar_hide (MNB_TOOLBAR (priv->toolbar));
 
       if (type == META_COMP_WINDOW_NORMAL)
         {
@@ -1695,14 +1674,36 @@ map (MutterPlugin *plugin, MutterWindow *mcw)
 
       /*
        * Move application window to a new workspace, if appropriate.
-       * (We only move applications, i.e., _NET_WM_WINDOW_TYPE_NORMAL that
-       * were start up with SN. We explicitely exclude modal windows and
-       * windows that are transient.)
+       *
+       *  - only move windows when in netbook mode,
+       *
+       *  - always move windows that set moblin-on-new-workspace hint to yes,
+       *
+       *  - never move windows that set moblin-on-new-workspace hint to no,
+       *
+       *  - apply heuristics when hint is not set:
+       *     - only move applications, i.e., _NET_WM_WINDOW_TYPE_NORMAL,
+       *     - exclude modal windows and windows that are transient.
        */
-      if (type == META_COMP_WINDOW_NORMAL  &&
-          !meta_window_is_modal (mw)       &&
-          meta_window_get_startup_id (mw) &&
-          meta_window_get_transient_for_as_xid (mw) == None)
+      if (priv->netbook_mode)
+        {
+          MnbThreeState state =
+            moblin_netbook_mutter_hints_on_new_workspace (mw);
+
+          if (state == MNB_STATE_YES)
+            {
+              move_window = TRUE;
+            }
+          else if (state == MNB_STATE_UNSET)
+            {
+              move_window =
+                (type == META_COMP_WINDOW_NORMAL  &&
+                 !meta_window_is_modal (mw)       &&
+                 meta_window_get_transient_for_as_xid (mw) == None);
+            }
+        }
+
+      if (move_window)
         {
           MetaDisplay *display = meta_screen_get_display (screen);
           guint32      timestamp;
@@ -1773,6 +1774,25 @@ map (MutterPlugin *plugin, MutterWindow *mcw)
 static void
 destroy (MutterPlugin *plugin, MutterWindow *mcw)
 {
+  MoblinNetbookPluginPrivate *priv = MOBLIN_NETBOOK_PLUGIN (plugin)->priv;
+  MetaCompWindowType          type;
+  Window                      xwin;
+
+  type = mutter_window_get_window_type (mcw);
+  xwin = mutter_window_get_x_window (mcw);
+
+  if (type == META_COMP_WINDOW_DOCK)
+    {
+      MnbPanel   *panel;
+      MnbToolbar *toolbar = MNB_TOOLBAR (priv->toolbar);
+
+      if ((panel = mnb_toolbar_find_panel_for_xid (toolbar, xwin)))
+        {
+          mnb_panel_oop_hide_animate (MNB_PANEL_OOP (panel), mcw);
+          return;
+        }
+    }
+
   handle_window_destruction (mcw, plugin);
   mutter_plugin_effect_completed (plugin, mcw, MUTTER_PLUGIN_DESTROY);
 }
@@ -1880,26 +1900,39 @@ xevent_filter (MutterPlugin *plugin, XEvent *xev)
 {
   MoblinNetbookPluginPrivate *priv = MOBLIN_NETBOOK_PLUGIN (plugin)->priv;
   MnbToolbar                 *toolbar = MNB_TOOLBAR (priv->toolbar);
-  NbtkWidget                 *switcher;
 
-  switcher = mnb_toolbar_get_switcher (toolbar);
-
-  if (switcher && mnb_switcher_handle_xevent (MNB_SWITCHER (switcher), xev))
-    return TRUE;
-
-  if (xev->type == KeyPress || xev->type == KeyRelease)
+  /*
+   * Avoid any unnecessary procesing here, as this function is called all the
+   * time.
+   *
+   * We ourselves only need to do extra work for Key events, and the Switcher
+   * only cares about Key and Pointer events.
+   */
+  if (xev->type == KeyPress      ||
+      xev->type == KeyRelease    ||
+      xev->type == ButtonPress   ||
+      xev->type == ButtonRelease ||
+      xev->type == MotionNotify)
     {
-      MetaScreen   *screen  = mutter_plugin_get_screen (plugin);
-      ClutterActor *stage   = mutter_get_stage_for_screen (screen);
-      Window        xwin;
+      MnbSwitcher *switcher = MNB_SWITCHER (mnb_toolbar_get_switcher (toolbar));
 
-      /*
-       * We only get key events on the no-focus window, but for clutter we
-       * need to pretend they come from the stage window.
-       */
-      xwin = clutter_x11_get_stage_window (CLUTTER_STAGE (stage));
+      if (switcher && mnb_switcher_handle_xevent (switcher, xev))
+        return TRUE;
 
-      xev->xany.window = xwin;
+      if (xev->type == KeyPress || xev->type == KeyRelease)
+        {
+          MetaScreen   *screen = mutter_plugin_get_screen (plugin);
+          ClutterActor *stage  = mutter_get_stage_for_screen (screen);
+          Window        xwin;
+
+          /*
+           * We only get key events on the no-focus window, but for clutter we
+           * need to pretend they come from the stage window.
+           */
+          xwin = clutter_x11_get_stage_window (CLUTTER_STAGE (stage));
+
+          xev->xany.window = xwin;
+        }
     }
 
   return (clutter_x11_handle_event (xev) != CLUTTER_X11_FILTER_CONTINUE);
@@ -1961,7 +1994,7 @@ setup_focus_window (MutterPlugin *plugin)
                                      META_ATOM__NET_WM_WINDOW_TYPE_DOCK);
 
   attr.event_mask        = KeyPressMask | KeyReleaseMask | FocusChangeMask;
-  attr.override_redirect = True;
+  attr.override_redirect = False;
 
   xwin = XCreateWindow (xdpy,
                         RootWindow (xdpy,
@@ -2196,23 +2229,23 @@ setup_desktop_background (MutterPlugin *plugin, const gchar *filename)
   MoblinNetbookPluginPrivate *priv = MOBLIN_NETBOOK_PLUGIN (plugin)->priv;
   MetaScreen                 *screen = mutter_plugin_get_screen (plugin);
   gint                        screen_width, screen_height;
-  gboolean                    new_texture = FALSE;
+  ClutterActor               *new_texture;
+  ClutterActor               *old_texture = priv->desktop_tex;
 
   mutter_plugin_query_screen_size (MUTTER_PLUGIN (plugin),
                                    &screen_width, &screen_height);
 
   g_assert (filename);
 
-  if (!priv->desktop_tex)
-    {
-      new_texture = TRUE;
-      priv->desktop_tex = clutter_texture_new_from_file (filename, NULL);
-    }
-  else
-    clutter_texture_set_from_file (CLUTTER_TEXTURE (priv->desktop_tex),
-                                   filename, NULL);
+  new_texture = (ClutterActor*)
+    mx_texture_cache_get_texture (mx_texture_cache_get_default (), filename);
 
-  if (priv->desktop_tex == NULL)
+  priv->desktop_tex = new_texture;
+
+  if (old_texture)
+    clutter_actor_destroy (old_texture);
+
+  if (!new_texture)
     {
       g_warning ("Failed to load '%s', No tiled desktop image", filename);
     }
@@ -2220,10 +2253,16 @@ setup_desktop_background (MutterPlugin *plugin, const gchar *filename)
     {
       ClutterActor *stage = mutter_get_stage_for_screen (screen);
 
-      clutter_actor_set_size (priv->desktop_tex, screen_width, screen_height);
+      if (clutter_texture_get_pixel_format (CLUTTER_TEXTURE (new_texture)) &
+          COGL_A_BIT)
+        {
+          g_warning ("Desktop background '%s' has alpha channel", filename);
+        }
+
+      clutter_actor_set_size (new_texture, screen_width, screen_height);
 
 #if !USE_SCALED_BACKGROUND
-      g_object_set (priv->desktop_tex,
+      g_object_set (new_texture,
                     "repeat-x", TRUE,
                     "repeat-y", TRUE,
                     NULL);
@@ -2231,11 +2270,10 @@ setup_desktop_background (MutterPlugin *plugin, const gchar *filename)
 
       if (new_texture)
         {
-          clutter_container_add_actor (CLUTTER_CONTAINER (stage),
-                                       priv->desktop_tex);
-          clutter_actor_lower_bottom (priv->desktop_tex);
+          clutter_container_add_actor (CLUTTER_CONTAINER (stage), new_texture);
+          clutter_actor_lower_bottom (new_texture);
 
-          g_signal_connect (priv->desktop_tex, "paint",
+          g_signal_connect (new_texture, "paint",
                             G_CALLBACK (desktop_background_paint),
                             plugin);
         }
@@ -2269,10 +2307,8 @@ desktop_background_init (MutterPlugin *plugin)
   MoblinNetbookPluginPrivate *priv = MOBLIN_NETBOOK_PLUGIN (plugin)->priv;
   GError *error = NULL;
 
-  priv->gconf_client = gconf_client_get_default ();
-
   gconf_client_add_dir (priv->gconf_client,
-                        KEY_DIR,
+                        BG_KEY_DIR,
                         GCONF_CLIENT_PRELOAD_NONE,
                         &error);
 
@@ -2354,7 +2390,7 @@ moblin_netbook_unstash_window_focus (MutterPlugin *plugin, guint32 timestamp)
   MetaScreen                 *screen  = mutter_plugin_get_screen (plugin);
   MetaDisplay                *display = meta_screen_get_display (screen);
   MetaWindow                 *focus;
-  NbtkWidget                 *panel;
+  MnbPanel                   *panel;
 
   /*
    * First check if a panel is active; if it's an OOP panel, we focus the panel,
@@ -2364,9 +2400,9 @@ moblin_netbook_unstash_window_focus (MutterPlugin *plugin, guint32 timestamp)
 
   if (panel)
     {
-      if (MNB_IS_PANEL (panel))
+      if (MNB_IS_PANEL_OOP (panel))
         {
-          mnb_panel_focus ((MnbPanel*)panel);
+          mnb_panel_oop_focus ((MnbPanelOop*)panel);
           return;
         }
     }
@@ -2425,7 +2461,14 @@ moblin_netbook_set_lowlight (MutterPlugin *plugin, gboolean on)
 
   if (on && !active)
     {
-      gint screen_width, screen_height;
+      gint          screen_width, screen_height;
+      MetaScreen   *screen  = mutter_plugin_get_screen (plugin);
+      ClutterActor *stage   = mutter_get_stage_for_screen (screen);
+      Window        xwin;
+      guint32       timestamp;
+
+      xwin      = clutter_x11_get_stage_window (CLUTTER_STAGE (stage));
+      timestamp = clutter_x11_get_current_event_time ();
 
       mutter_plugin_query_screen_size (plugin, &screen_width, &screen_height);
 
@@ -2439,17 +2482,26 @@ moblin_netbook_set_lowlight (MutterPlugin *plugin, gboolean on)
       clutter_actor_lower (priv->lowlight, priv->notification_urgent);
       clutter_actor_show (priv->lowlight);
       active = TRUE;
+      mutter_plugin_begin_modal (plugin, xwin, None,
+                                 META_MODAL_POINTER_ALREADY_GRABBED,
+                                 timestamp);
+
       mnb_toolbar_set_disabled (MNB_TOOLBAR (priv->toolbar), TRUE);
     }
   else
     {
       if (active)
         {
+          guint32 timestamp;
+
+          timestamp = clutter_x11_get_current_event_time ();
+
           clutter_actor_hide (priv->lowlight);
           mnb_input_manager_remove_region (input_region);
           input_region = NULL;
           active = FALSE;
           mnb_toolbar_set_disabled (MNB_TOOLBAR (priv->toolbar), FALSE);
+          mutter_plugin_end_modal (plugin, timestamp);
         }
     }
 }
@@ -2723,7 +2775,8 @@ meta_display_window_demands_attention_cb (MetaDisplay *display,
 {
   MutterPlugin       *plugin = MUTTER_PLUGIN (data);
 
-  get_demands_attention_notification (plugin, mw, FALSE);
+  if (mw != meta_display_get_focus_window (display))
+    get_demands_attention_notification (plugin, mw, FALSE);
 }
 
 void
@@ -2754,3 +2807,52 @@ moblin_netbook_activate_window (MetaWindow *window)
       meta_workspace_activate_with_focus (workspace, window, timestamp);
     }
 }
+
+ClutterActor *
+moblin_netbook_get_toolbar (MutterPlugin *plugin)
+{
+  MoblinNetbookPluginPrivate *priv = MOBLIN_NETBOOK_PLUGIN (plugin)->priv;
+
+  return priv->toolbar;
+}
+
+gboolean
+moblin_netbook_activate_mutter_window (MutterWindow *mcw)
+{
+  MetaWindow     *window;
+  MetaWorkspace  *workspace;
+  MetaWorkspace  *active_workspace;
+  MetaScreen     *screen;
+  MetaDisplay    *display;
+  guint32         timestamp;
+
+  window           = mutter_window_get_meta_window (mcw);
+  screen           = meta_window_get_screen (window);
+  display          = meta_screen_get_display (screen);
+  workspace        = meta_window_get_workspace (window);
+  active_workspace = meta_screen_get_active_workspace (screen);
+
+  /*
+   * Make sure our stamp is recent enough.
+   */
+  timestamp = meta_display_get_current_time_roundtrip (display);
+
+  if (!active_workspace || (active_workspace == workspace))
+    {
+      meta_window_activate_with_workspace (window, timestamp, workspace);
+    }
+  else
+    {
+      meta_workspace_activate_with_focus (workspace, window, timestamp);
+    }
+
+  return TRUE;
+}
+
+gboolean
+moblin_netbook_use_netbook_mode (MutterPlugin *plugin)
+{
+  MoblinNetbookPluginPrivate *priv = MOBLIN_NETBOOK_PLUGIN (plugin)->priv;
+
+  return priv->netbook_mode;
+}
