@@ -25,12 +25,14 @@
 #include <clutter/x11/clutter-x11.h>
 
 #include "mnb-switcher-zone.h"
+#include "../mnb-panel.h"
 
 struct _MnbSwitcherZonePrivate
 {
   MnbSwitcher  *switcher;
 
   gint          index;
+  guint32       timestamp;
 
   ClutterActor *table;             /* the inner content area     */
   ClutterActor *label;             /* the label area             */
@@ -58,7 +60,7 @@ enum
   PROP_HAS_ITEMS
 };
 
-G_DEFINE_ABSTRACT_TYPE (MnbSwitcherZone, mnb_switcher_zone, NBTK_TYPE_TABLE);
+G_DEFINE_ABSTRACT_TYPE (MnbSwitcherZone, mnb_switcher_zone, MX_TYPE_TABLE);
 
 #define MNB_SWITCHER_ZONE_GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), MNB_TYPE_SWITCHER_ZONE,\
@@ -135,7 +137,7 @@ mnb_switcher_zone_set_property (GObject      *gobject,
         priv->label_text = g_value_dup_string (value);
 
         if (priv->text)
-          nbtk_label_set_text (NBTK_LABEL (priv->text), priv->label_text);
+          mx_label_set_text (MX_LABEL (priv->text), priv->label_text);
 
         if (priv->label_text)
           priv->text_set = TRUE;
@@ -199,9 +201,9 @@ mnb_switcher_zone_constructed (GObject *self)
 
   g_assert (priv->index >= 0);
 
-  priv->table = table = (ClutterActor*)nbtk_table_new ();
-  nbtk_table_add_actor (NBTK_TABLE (self), table, 1, 0);
-  nbtk_table_set_row_spacing (NBTK_TABLE (self), 4);
+  priv->table = table = (ClutterActor*)mx_table_new ();
+  mx_table_add_actor (MX_TABLE (self), table, 1, 0);
+  mx_table_set_row_spacing (MX_TABLE (self), 4);
 
   if (priv->active)
     state = MNB_SWITCHER_ZONE_ACTIVE;
@@ -210,7 +212,7 @@ mnb_switcher_zone_constructed (GObject *self)
   clutter_actor_set_name (table, zone_name);
 
   zone_class = klass->zone_class (zone);
-  nbtk_widget_set_style_class_name (NBTK_WIDGET (table), zone_class);
+  mx_stylable_set_style_class (MX_STYLABLE (table), zone_class);
 
   /*
    * Now construct the label area
@@ -235,18 +237,18 @@ mnb_switcher_zone_constructed (GObject *self)
       else
         s = g_strdup_printf ("%d", priv->index + 1);
 
-      priv->label = label = CLUTTER_ACTOR (nbtk_bin_new ());
-      priv->text  = text  = (ClutterActor*)nbtk_label_new (s);
+      priv->label = label = CLUTTER_ACTOR (mx_frame_new ());
+      priv->text  = text  = (ClutterActor*)mx_label_new (s);
 
-      nbtk_widget_set_style_class_name (NBTK_WIDGET (text), text_class);
+      mx_stylable_set_style_class (MX_STYLABLE (text), text_class);
       clutter_actor_set_name (CLUTTER_ACTOR (text), text_name);
 
-      nbtk_bin_set_child (NBTK_BIN (label), CLUTTER_ACTOR (text));
+      mx_bin_set_child (MX_BIN (label), CLUTTER_ACTOR (text));
 
-      nbtk_widget_set_style_class_name (NBTK_WIDGET (label), label_class);
+      mx_stylable_set_style_class (MX_STYLABLE (label), label_class);
       clutter_actor_set_name (CLUTTER_ACTOR (label), label_name);
 
-      nbtk_table_add_actor (NBTK_TABLE (self), label, 0, 0);
+      mx_table_add_actor (MX_TABLE (self), label, 0, 0);
       clutter_container_child_set (CLUTTER_CONTAINER (self),
                                    CLUTTER_ACTOR (label),
                                    "y-expand", FALSE,
@@ -335,6 +337,29 @@ mnb_switcher_zone_text_style (MnbSwitcherZone     *zone,
   return text_name;
 }
 
+static void
+mnb_switcher_zone_hide_completed_cb (MnbSwitcher     *panel,
+                                     MnbSwitcherZone *zone)
+{
+  MnbSwitcherZonePrivate *priv      = zone->priv;
+  MutterPlugin           *plugin    = moblin_netbook_get_plugin_singleton ();
+  MetaScreen             *screen    = mutter_plugin_get_screen (plugin);
+  MetaWorkspace          *workspace =
+    meta_screen_get_workspace_by_index (screen, priv->index);
+
+  g_signal_handlers_disconnect_by_func (panel,
+                                        mnb_switcher_zone_hide_completed_cb,
+                                        zone);
+
+  if (!workspace)
+    {
+      g_warning ("No workspace specified, %s: %d", __FILE__, __LINE__);
+      return;
+    }
+
+  meta_workspace_activate (workspace, priv->timestamp);
+}
+
 /*
  * Handling of button release on the zone -- we switch to the corresponding
  * workspace.
@@ -344,27 +369,18 @@ mnb_switcher_zone_button_release (ClutterActor       *actor,
                                   ClutterButtonEvent *event)
 {
   MnbSwitcherZonePrivate *priv      = MNB_SWITCHER_ZONE (actor)->priv;
-  MutterPlugin           *plugin    = moblin_netbook_get_plugin_singleton ();
-  MetaScreen             *screen    = mutter_plugin_get_screen (plugin);
-  guint32                 timestamp = clutter_x11_get_current_event_time ();
-  MetaWorkspace          *workspace;
 
   if (mnb_switcher_get_dnd_in_progress (priv->switcher))
     return FALSE;
 
-  workspace = meta_screen_get_workspace_by_index (screen, priv->index);
+  priv->timestamp = clutter_x11_get_current_event_time ();
 
-  if (!workspace)
-    {
-      g_warning ("No workspace specified, %s:%d\n", __FILE__, __LINE__);
-      return FALSE;
-    }
-
-  mnb_drop_down_hide_with_toolbar (MNB_DROP_DOWN (priv->switcher));
+  mnb_panel_hide_with_toolbar (MNB_PANEL (priv->switcher));
 
   mnb_switcher_end_kbd_grab (priv->switcher);
 
-  meta_workspace_activate (workspace, timestamp);
+  g_signal_connect (priv->switcher, "hide-completed",
+                    G_CALLBACK (mnb_switcher_zone_hide_completed_cb), actor);
 
   return FALSE;
 }
@@ -710,7 +726,7 @@ mnb_switcher_zone_set_index (MnbSwitcherZone *zone, gint index)
     {
       gchar *s = g_strdup_printf ("%d", index + 1);
 
-      nbtk_label_set_text (NBTK_LABEL (priv->text), s);
+      mx_label_set_text (MX_LABEL (priv->text), s);
       g_free (s);
     }
 
@@ -721,12 +737,12 @@ mnb_switcher_zone_set_index (MnbSwitcherZone *zone, gint index)
  * Returns the content area of this table, i.e., where the items go; this can
  * be used by any subclasses to manipulate their contents.
  */
-NbtkTable *
+MxTable *
 mnb_switcher_zone_get_content_area (MnbSwitcherZone *zone)
 {
   MnbSwitcherZonePrivate *priv = zone->priv;
 
-  return NBTK_TABLE (priv->table);
+  return MX_TABLE (priv->table);
 }
 
 gboolean
