@@ -2,7 +2,7 @@
 
 /*
  * Moblin Netbook
- * Copyright © 2009, Intel Corporation.
+ * Copyright © 2009, 2010, Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,7 +32,7 @@
 #include <display.h>
 #include <clutter/clutter.h>
 #include <clutter/x11/clutter-x11.h>
-#include <nbtk/nbtk.h>
+#include <mx/mx.h>
 #include <keybindings.h>
 
 #include <glib/gi18n.h>
@@ -45,7 +45,13 @@ static void mnb_switcher_tooltip_weak_notify (gpointer data, GObject *obj);
  * MnbSwitcher is an MnbDropDown subclass implementing the Zones panel of the
  * shell.
  */
-G_DEFINE_TYPE (MnbSwitcher, mnb_switcher, MNB_TYPE_DROP_DOWN)
+static void mnb_panel_iface_init (MnbPanelIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (MnbSwitcher,
+                         mnb_switcher,
+                         MNB_TYPE_DROP_DOWN,
+                         G_IMPLEMENT_INTERFACE (MNB_TYPE_PANEL,
+                                                mnb_panel_iface_init));
 
 #define MNB_SWITCHER_GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), MNB_TYPE_SWITCHER, MnbSwitcherPrivate))
@@ -129,7 +135,7 @@ mnb_switcher_enable_new_workspace (MnbSwitcher *switcher, MnbSwitcherZone *zone)
    * If the application is the only child in its zone, the new zone remains
    * disabled.
    *
-   * Would like to be able to use here nbtk_table_get_row_count(), but it
+   * Would like to be able to use here mx_table_get_row_count(), but it
    * is unreliable.
    */
 
@@ -207,7 +213,7 @@ mnb_switcher_show (ClutterActor *self)
   MetaScreen         *screen = mutter_plugin_get_screen (priv->plugin);
   gint                ws_count, active_ws;
   gint                i, screen_width, screen_height;
-  NbtkWidget         *table;
+  ClutterActor       *table;
   ClutterActor       *toolbar;
   gint                apps_count = 0xffff; /* inital value > 1 */
 
@@ -243,10 +249,10 @@ mnb_switcher_show (ClutterActor *self)
   priv->last_workspaces = g_list_copy (meta_screen_get_workspaces (screen));
 
   /* create the contents */
-  table = nbtk_table_new ();
+  table = mx_table_new ();
   priv->table = table;
-  nbtk_table_set_row_spacing (NBTK_TABLE (table), 4);
-  nbtk_table_set_col_spacing (NBTK_TABLE (table), 7);
+  mx_table_set_row_spacing (MX_TABLE (table), 4);
+  mx_table_set_col_spacing (MX_TABLE (table), 7);
 
   clutter_actor_set_name (CLUTTER_ACTOR (table), "switcher-table");
 
@@ -293,41 +299,42 @@ mnb_switcher_show (ClutterActor *self)
 
       if (!apps_count)
         {
-          NbtkWidget         *table = priv->table;
-          ClutterActor       *bin;
-          NbtkWidget         *label;
+          ClutterActor         *table = priv->table;
+          ClutterActor     *bin;
+          ClutterActor     *label;
 
-          bin = CLUTTER_ACTOR (nbtk_bin_new ());
-          label = nbtk_label_new (_("No Zones yet"));
+          bin = CLUTTER_ACTOR (mx_frame_new ());
+          label = mx_label_new (_("No Zones yet"));
 
-          nbtk_widget_set_style_class_name (label, "workspace-title-label");
+          mx_stylable_set_style_class (MX_STYLABLE (label),
+                                       "workspace-title-label");
 
-          nbtk_bin_set_child (NBTK_BIN (bin), CLUTTER_ACTOR (label));
+          mx_bin_set_child (MX_BIN (bin), CLUTTER_ACTOR (label));
 
           clutter_actor_set_name (CLUTTER_ACTOR (bin),
                                   "workspace-title-active");
 
-          nbtk_widget_set_style_class_name (NBTK_WIDGET (bin),
-                                            "workspace-title");
+          mx_stylable_set_style_class (MX_STYLABLE (bin),
+                                       "workspace-title");
 
-          nbtk_table_add_actor (NBTK_TABLE (table), bin, 0, 0);
+          mx_table_add_actor (MX_TABLE (table), bin, 0, 0);
           clutter_container_child_set (CLUTTER_CONTAINER (table),
                                        CLUTTER_ACTOR (bin),
                                        "y-expand", FALSE,
                                        "x-fill", TRUE,
                                        NULL);
 
-          bin = CLUTTER_ACTOR (nbtk_bin_new ());
-          label = nbtk_label_new (_("Applications you’re using will show up here. You will be able to switch and organize them to your heart's content."));
+          bin = CLUTTER_ACTOR (mx_frame_new ());
+          label = mx_label_new (_("Applications you’re using will show up here. You will be able to switch and organize them to your heart's content."));
           clutter_actor_set_name ((ClutterActor *)label, "workspace-no-wins-label");
 
-          nbtk_bin_set_child (NBTK_BIN (bin), CLUTTER_ACTOR (label));
-          nbtk_bin_set_alignment (NBTK_BIN (bin), NBTK_ALIGN_START,
-                                  NBTK_ALIGN_MIDDLE);
+          mx_bin_set_child (MX_BIN (bin), CLUTTER_ACTOR (label));
+          mx_bin_set_alignment (MX_BIN (bin), MX_ALIGN_START,
+                                  MX_ALIGN_MIDDLE);
           clutter_actor_set_name (CLUTTER_ACTOR (bin),
                                   "workspace-no-wins-bin");
 
-          nbtk_table_add_actor (NBTK_TABLE (table), bin, 1, 0);
+          mx_table_add_actor (MX_TABLE (table), bin, 1, 0);
           clutter_container_child_set (CLUTTER_CONTAINER (table),
                                        CLUTTER_ACTOR (bin),
                                        "y-expand", FALSE,
@@ -348,7 +355,7 @@ mnb_switcher_show (ClutterActor *self)
         active = TRUE;
 
       zone = mnb_switcher_zone_apps_new (switcher, active, i);
-      nbtk_table_add_actor (NBTK_TABLE (table), CLUTTER_ACTOR (zone), 0, i);
+      mx_table_add_actor (MX_TABLE (table), CLUTTER_ACTOR (zone), 0, i);
       mnb_switcher_zone_apps_populate (zone, dnd);
       g_object_set (zone, "enabled", TRUE, NULL);
 
@@ -377,7 +384,7 @@ mnb_switcher_show (ClutterActor *self)
      */
     g_object_set (new_ws, "enabled", FALSE, NULL);
 
-    nbtk_table_add_actor (NBTK_TABLE (table), CLUTTER_ACTOR (new_ws), 0,
+    mx_table_add_actor (MX_TABLE (table), CLUTTER_ACTOR (new_ws), 0,
                           ws_count);
 
     clutter_container_child_set (CLUTTER_CONTAINER (table),
@@ -423,11 +430,11 @@ ClutterActor *
 mnb_switcher_append_app_zone (MnbSwitcher *switcher, gint index)
 {
   MnbSwitcherPrivate *priv  = switcher->priv;
-  NbtkWidget         *table = priv->table;
+  ClutterActor         *table = priv->table;
   ClutterActor       *zone;
 
   zone = (ClutterActor*)mnb_switcher_zone_apps_new (switcher, FALSE, index);
-  nbtk_table_add_actor (NBTK_TABLE (table), zone, 0, index);
+  mx_table_add_actor (MX_TABLE (table), zone, 0, index);
   g_object_set (zone, "enabled", TRUE, NULL);
 
   /*
@@ -457,7 +464,19 @@ mnb_switcher_hide (ClutterActor *self)
    * item dispose vfunction, but if something breaks and an item leaks, we want
    * to make sure that we at least have no left over tooltips on screen.
    */
-  mnb_switcher_hide_tooltip ((MnbSwitcher*)self);
+  if (priv->active_tooltip)
+    {
+      g_object_weak_unref (G_OBJECT (priv->active_tooltip),
+                           mnb_switcher_tooltip_weak_notify, self);
+
+      /*
+       * We must use the plain clutter_actor_hide() here, since
+       * mx_tooltip_hide() triggers animation, which will not finish until after
+       * the tooltip has been disposed of.
+       */
+      clutter_actor_hide ((ClutterActor*)priv->active_tooltip);
+      priv->active_tooltip = NULL;
+    }
 
   if (priv->show_completed_id)
     {
@@ -617,7 +636,7 @@ mnb_switcher_n_workspaces_notify (MetaScreen *screen,
 
           remove_count++;
 
-          /* NbtkTable provides no API to get a child at given position, so we
+          /* MxTable provides no API to get a child at given position, so we
            * have to iterate the child list, querying the child column property
            */
           while (t)
@@ -749,7 +768,7 @@ mnb_switcher_init (MnbSwitcher *self)
   mnb_switcher_setup_metacity_keybindings (self);
 }
 
-NbtkWidget*
+ClutterActor*
 mnb_switcher_new (MutterPlugin *plugin)
 {
   MnbSwitcher *switcher;
@@ -763,7 +782,7 @@ mnb_switcher_new (MutterPlugin *plugin)
                            "mutter-plugin", plugin,
                            NULL);
 
-  return NBTK_WIDGET (switcher);
+  return (ClutterActor*)switcher;
 }
 
 /*
@@ -861,13 +880,8 @@ mnb_switcher_select_zone (MnbSwitcher *switcher, MnbSwitcherZone *zone)
     }
 }
 
-/*
- * Activates the window currently selected in the switcher.
- */
-void
-mnb_switcher_activate_selection (MnbSwitcher *switcher,
-                                 gboolean     close,
-                                 guint        timestamp)
+static void
+mnb_switcher_activate_selection_cb (MnbSwitcher *switcher)
 {
   MnbSwitcherPrivate *priv = switcher->priv;
   MnbSwitcherItem    *item = priv->selected_item;
@@ -882,9 +896,34 @@ mnb_switcher_activate_selection (MnbSwitcher *switcher,
       g_warning (G_STRLOC " Nothing to activate");
       return;
     }
+}
 
+static void
+mnb_switcher_hide_completed_cb (MnbSwitcher *panel,
+                                MnbSwitcher *switcher)
+{
+  g_signal_handlers_disconnect_by_func (panel,
+                                        mnb_switcher_hide_completed_cb,
+                                        switcher);
+  mnb_switcher_activate_selection_cb (switcher);
+}
+
+/*
+ * Activates the window currently selected in the switcher.
+ */
+void
+mnb_switcher_activate_selection (MnbSwitcher *switcher,
+                                 gboolean     close,
+                                 guint        timestamp)
+{
   if (close)
-    mnb_drop_down_hide_with_toolbar (MNB_DROP_DOWN (switcher));
+    {
+      mnb_panel_hide_with_toolbar (MNB_PANEL (switcher));
+      g_signal_connect (switcher, "hide-completed",
+                        G_CALLBACK (mnb_switcher_hide_completed_cb), switcher);
+    }
+  else
+    mnb_switcher_activate_selection_cb (switcher);
 }
 
 /*
@@ -1133,7 +1172,7 @@ mnb_switcher_tooltip_weak_notify (gpointer data, GObject *obj)
 {
   MnbSwitcherPrivate *priv = MNB_SWITCHER (data)->priv;
 
-  if (priv->active_tooltip == (NbtkTooltip*)obj)
+  if (priv->active_tooltip == (MxTooltip*)obj)
     priv->active_tooltip = NULL;
 }
 
@@ -1149,7 +1188,7 @@ mnb_switcher_hide_tooltip (MnbSwitcher *switcher)
     {
       g_object_weak_unref (G_OBJECT (priv->active_tooltip),
                            mnb_switcher_tooltip_weak_notify, switcher);
-      nbtk_tooltip_hide (NBTK_TOOLTIP (priv->active_tooltip));
+      mx_tooltip_hide (MX_TOOLTIP (priv->active_tooltip));
       priv->active_tooltip = NULL;
     }
 }
@@ -1158,7 +1197,7 @@ mnb_switcher_hide_tooltip (MnbSwitcher *switcher)
  * Shows the provided tooltip, hiding any previously visible tooltips.
  */
 void
-mnb_switcher_show_tooltip (MnbSwitcher *switcher, NbtkTooltip *tooltip)
+mnb_switcher_show_tooltip (MnbSwitcher *switcher, MxTooltip *tooltip)
 {
   MnbSwitcherPrivate *priv = switcher->priv;
 
@@ -1166,16 +1205,16 @@ mnb_switcher_show_tooltip (MnbSwitcher *switcher, NbtkTooltip *tooltip)
     {
       g_object_weak_unref (G_OBJECT (priv->active_tooltip),
                            mnb_switcher_tooltip_weak_notify, switcher);
-      nbtk_tooltip_hide (NBTK_TOOLTIP (priv->active_tooltip));
+      mx_tooltip_hide (MX_TOOLTIP (priv->active_tooltip));
       priv->active_tooltip = NULL;
     }
 
   if (tooltip)
     {
-      nbtk_tooltip_show (tooltip);
+      mx_tooltip_show (tooltip);
       g_object_weak_ref (G_OBJECT (tooltip),
                          mnb_switcher_tooltip_weak_notify, switcher);
-      priv->active_tooltip = NBTK_TOOLTIP (tooltip);
+      priv->active_tooltip = MX_TOOLTIP (tooltip);
     }
 }
 
@@ -1214,18 +1253,58 @@ mnb_switcher_end_kbd_grab (MnbSwitcher *switcher)
     }
 }
 
-void
-mnb_switcher_set_size (MnbSwitcher *switcher, guint width, guint height)
+static void
+mnb_switcher_set_size (MnbPanel *panel, guint width, guint height)
 {
-  MnbSwitcherPrivate *priv = switcher->priv;
+  MnbSwitcherPrivate *priv = MNB_SWITCHER (panel)->priv;
+
+  if (!priv->table)
+    return;
 
   if (!priv->empty)
     clutter_actor_set_size (CLUTTER_ACTOR (priv->table),
                             width - TOOLBAR_X_PADDING * 2,
-                            height - TOOLBAR_HEIGHT * 1.5 - 4 +
-                            MNB_DROP_DOWN_SHADOW_HEIGHT);
+                            height - TOOLBAR_HEIGHT - 8);
   else
     clutter_actor_set_width (CLUTTER_ACTOR (priv->table),
                              width - TOOLBAR_X_PADDING * 2);
 }
 
+static const gchar *
+mnb_switcher_get_name (MnbPanel *panel)
+{
+  return "zones";
+}
+
+static const gchar *
+mnb_switcher_get_tooltip (MnbPanel *panel)
+{
+  return _("zones");
+}
+
+static const gchar *
+mnb_switcher_get_button_style (MnbPanel *panel)
+{
+  return "zones-button";
+}
+
+static gboolean
+mnb_switcher_is_mapped (MnbPanel *panel)
+{
+  g_return_val_if_fail (MNB_IS_SWITCHER (panel), FALSE);
+
+  return CLUTTER_ACTOR_IS_MAPPED (panel);
+}
+
+static void
+mnb_panel_iface_init (MnbPanelIface *iface)
+{
+  iface->get_name         = mnb_switcher_get_name;
+  iface->get_tooltip      = mnb_switcher_get_tooltip;
+  iface->get_button_style = mnb_switcher_get_button_style;
+
+  iface->set_size         = mnb_switcher_set_size;
+
+  iface->is_mapped        = mnb_switcher_is_mapped;
+}
+
